#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Barcode Sample (Assembly)
*
*  Description: Contains code to demonstrate how to capture Barcodes using the
*               MatrixNetSdk assembly.  It also uses the SymbologyDialog and
*               SelectCommPortDialog components which are included with the
*               .NET SDK.
*
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.SymbologyComponent;
using Honeywell.WinForms.HHI.CommPortComponent;

#endregion

namespace BarcodeSample_Assembly_
{
    public partial class BarcodeSample : Form
    {
        #region Constructor

        public BarcodeSample()
        {
            InitializeComponent();

            // Only do connect if not running in the Visual Studio 2005 Designer
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // Hook the on barcode data event to capture asynchronous barcodes (like from a hardware trigger).
                imager.barcodeEvent += new ImagerSdk.BarcodeEventHandler(OnBarcodeEvent);
                // Register for the up arrow as a hotkey
                imager.RegisterGlobalHotKey(Keys.Up, HotKeyModifiers.MOD_NONE);
                imager.globalHotKeyEvent += new ImagerSdk.HotKeyEventHandler(OnHotKey);
            }
        }

        #endregion

        #region Private Members

        private const int decodeTimeout = 10000;
        private const int WM_KEYUP = 101;
        private ImagerSdk imager = new ImagerSdk();
        private eCommPort commPort = eCommPort.LAST_COM_PORT;
        private CommPorts commPortsCls = new CommPorts();
        private SymbologyDialog symbologyDlg = new SymbologyDialog();
        private SelectCommPortDialog commPortDlg = new SelectCommPortDialog();
        private DecodeData barcodeData = new DecodeData();
        private bool bCapturingBarcode = false;
        private bool bHotKeyDown = true;

        #endregion

        #region Private Methods

        // Have we tried to set the comm port at least once.
        private bool bCommPortSet = false;
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void BarcodeSample_Activated(object sender, EventArgs e)
        {
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                if (!bCommPortSet)
                {
                    bCommPortSet = true;
                    // Set the default commPort
                    commPort = commPortsCls.DefaultCommPort;
                    // Load the current port and intialize the filter list into the select comm port dialog.
                    selectCommPortDialog1.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                    selectCommPortDialog1.commPort = commPort;
                    // If no default port defined, get one from the user.
                    // If no default port defined, get one from the user.
                    if (selectCommPortDialog1.commPort == eCommPort.LAST_COM_PORT)
                    {
                        Refresh();
                        selectCommPortToolStripMenuItem_Click(this, new EventArgs());
                    }
                    // If one is defined now, connect.
                    if (selectCommPortDialog1.commPort != eCommPort.LAST_COM_PORT)
                    {
                        ConnectTimer.Interval = 100;
                        ConnectTimer.Enabled = true;
                    }
                    // We're done with the commPorts class so we can release the memory.
                    commPortsCls.Dispose();
                }
            }
        }

        private void DisplayBarcodeData(DecodeData data)
        {
            textBoxBarCode.SelectionStart = textBoxBarCode.Text.Length;
            textBoxBarCode.ScrollToCaret();
            textBoxBarCode.Text += imager.TranslateByteArray(data.rawMessage, data.length, true, true, false);
            textBoxBarCode.Text += "\r\n\r\n";
        }

        private void OnBarcodeEvent(object obj, CBarcodeEvent barcodeEvent)
        {
            if (barcodeEvent.EventType == EventType.ET_BARCODE)
            {
                bCapturingBarcode = false;
                if (barcodeEvent.nResult != ImagerResult.RESULT_SUCCESS)
                    textBoxStatus.Text = imager.GetErrorMessage(barcodeEvent.nResult);
                else
                    DisplayBarcodeData(barcodeEvent.BarcodeData);
            }
        }

        private void OnHotKey(object obj, CHotKey hotKey)
        {
            if (!bCapturingBarcode && !bHotKeyDown)
            {
                bCapturingBarcode = bHotKeyDown = true;
                try
                {
                    textBoxStatus.Text = "Capturing Barcode...";
                    imager.CaptureBarcodeNoWait(decodeTimeout);
                }
                catch (ImagerException ex)
                {
                    bCapturingBarcode = false;
                    textBoxStatus.Text = ex.Message;
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void selectCommPortToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Set the starting commPort.
            commPortDlg.commPort = commPort;
            // Show the dialog. It will return OK if the apply button is selected.
            if (commPortDlg.ShowDialog() == DialogResult.OK)
            {
                // If this is a new port, prepare to disconnect from current then reconnect.
                if (commPort != commPortDlg.commPort)
                {
                    // Set the new port
                    commPort = commPortDlg.commPort;
                    // Recycle the connection to disconnect from the current port and connect to the new port.
                    ConnectTimer.Enabled = true;
                }
            }
        }

        private void recycleConnectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConnectTimer.Enabled = true;
        }

        private void configureSymbologiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Set the symbology dialog location so it's centered on this form.
            symbologyDlg.Location = new Point((Location.X + ((Width - symbologyDlg.DialogWidth) / 2)), (Location.Y + ((Height - symbologyDlg.DialogHeight) / 2)));
            // Run the dialog.
            symbologyDlg.ShowDialog();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        private void buttonCaptureBarcode_Click(object sender, EventArgs e)
        {
            try
            {
                textBoxStatus.Text = "Capturing Barcode...";
                Refresh();
                bCapturingBarcode = true;
                imager.CaptureBarcode(barcodeData, decodeTimeout);
                bCapturingBarcode = false;
                DisplayBarcodeData(barcodeData);
            }
            catch (ImagerException ex)
            {
                textBoxStatus.Text = ex.Message;
            }
        }

        private void buttonClearView_Click(object sender, EventArgs e)
        {
            textBoxBarCode.Text = "";
        }

        private void ConnectTimer_Tick(object sender, EventArgs e)
        {
            ConnectTimer.Enabled = false;
            try
            {
                string str = "Connecting...";
                if (imager.IsConnected())
                {
                    textBoxStatus.Text = "Disconnecting...";
                    Refresh();
                    imager.Disconnect();
                    System.Threading.Thread.Sleep(1000);
                    str = "Reconnecting...";
                }
                textBoxStatus.Text = str;
                Refresh();
                imager.Connect(commPort);
                textBoxStatus.Text = "Connected On " + commPort.ToString();
            }
            catch (ImagerException ex)
            {
                textBoxStatus.Text += ("Failed" + ex.Message);
            }
        }

        // Make sure to set KeyPreview in the designer to true.
        private void BarcodeSample_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Up)
            {
                if (bHotKeyDown)
                {
                    bHotKeyDown = false;
                    if (bCapturingBarcode)
                        imager.CancelIO();
                }
            }
        }
        #endregion
    }
}
