#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Barcode Sample (Control)
*
*  Description: Contains code to demonstrate how to capture barcodes using the
*               Matrix DecodeControl.  It also uses the SymbologyDialog and
*               SelectCommPortDialog components which are included with the
*               .NET SDK.
*
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Honeywell.WinForms.HHI.DecodeControl;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.CommPortComponent;

#endregion

namespace BarcodeSample_Control_
{
    public partial class BarcodeSample : Form
    {
        public BarcodeSample()
        {
            InitializeComponent();
        }
        /// <summary>CommPorts class instance used to determine whether port is a serial port or not</summary>
        private CommPorts commPorts = new CommPorts();
        /// <summary>Is current port a serial port?.</summary>
        private bool bIsSerialPort = false;
        /// <summary>Barcode capture in progress?.</summary>
        private bool bBarcodeCapture = false;
        // Have we tried to set the comm port at least once.
        private bool bCommPortSet = false;
        // Local copy of comm port
        private eCommPort commPort = eCommPort.LAST_COM_PORT;
        // Component used to select a comm port
        private CommPorts commPortsCls = new CommPorts();
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void BarcodeSample_Activated(object sender, EventArgs e)
        {
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                if (!bCommPortSet)
                {
                    bCommPortSet = true;
                    // Set the default commPort
                    commPort = commPortsCls.DefaultCommPort; ;
                    // Load the current port and intialize the filter list into the select comm port dialog.
                    selectCommPortDialog1.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                    selectCommPortDialog1.commPort = commPort;
                    // If no default port defined, get one from the user.
                    if (selectCommPortDialog1.commPort == eCommPort.LAST_COM_PORT)
                    {
                        Refresh();
                        OnSelectCommPort(this, new EventArgs());
                    }
                    // If one is defined now, connect.
                    if (selectCommPortDialog1.commPort != eCommPort.LAST_COM_PORT)
                    {
                        connectTimer.Interval = 100;
                        connectTimer.Enabled = true;
                    }
                    // We're done with the commPorts class so we can release the memory.
                    commPortsCls.Dispose();
                }
            }
        }
        /// <summary>
        /// Make sure to disconnect when exiting the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnClosing(object sender, FormClosingEventArgs e)
        {
            decodeControl1.Disconnect();
        }
        /// <summary>
        /// Capture barcode button selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCapture(object sender, EventArgs e)
        {
            // Make sure we're not already capturing a barcode.
            if (!bBarcodeCapture)
            {
                // Make SDK calls inside try blocks. SDK methods will throw a ImagerException
                // on method failure.
                try
                {
                    decodeControl1.CaptureBarcodeNoWait();
                    bBarcodeCapture = true;
                }
                catch (ImagerException ex)
                {
                    decodeControl1.PlaySounds(false);
                    StatuLabel.Text = ex.Message;
                    bBarcodeCapture = false;
                }
            }
        }
        /// <summary>
        /// Clear text from text box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnClearView(object sender, EventArgs e)
        {
            decodeControl1.Text = "";
        }
        /// <summary>
        /// A event of some type has been received from the imager.
        /// </summary>
        /// <param name="theDecoder"></param>
        /// <param name="dataEvent"></param>
        public void ImagerEvent(object theDecoder, DataEvent dataEvent)
        {
            // Is this a barcode event?  If so, display symbology type, no decode or error message as appropriate.
            if (dataEvent.EventType == EventType.ET_BARCODE)
            {
                if (dataEvent.nResult == ImagerResult.RESULT_SUCCESS)
                    StatusLabel.Text = SymbologyConfig.SymbologyNameFromSymInfo(decodeControl1.barcodeData.codeID, decodeControl1.barcodeData.symLetter, decodeControl1.barcodeData.symModifier);
                else if (dataEvent.nResult == ImagerResult.RESULT_ERR_NODECODE)
                    this.StatusLabel.Text = "No Decode";
                else
                    this.StatusLabel.Text = decodeControl1.imgrCaps.GetErrorString(dataEvent.nResult);
                // Reset barcode capture in progress flag.
                bBarcodeCapture = false;
            }
        }
        /// <summary>
        /// A timer is used to start connect so that the gui has time to redraw on app start or recycle.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        private void TimerEvent(Object obj, EventArgs args)
        {
            StatusLabel.Text = "Connecting To Imager...";
            connectTimer.Enabled = false;
            try
            {
                if (ConnectToImager())
                    DisplayFirmwareRevisions();
            }
            catch (ImagerException ex)
            {
                this.StatusLabel.Text = "Connect Failed: " + ex.Message;
            }
        }
        /// <summary>
        /// This method displays the imager software information.
        /// </summary>
        private void DisplayFirmwareRevisions()
        {
            VersionData verInfo = new VersionData();

            decodeControl1.AddText(string.Format("{0}\r\n", verInfo.apiRev), false);
            decodeControl1.AddText(string.Format("Firmware Rev: {0}\r\n", verInfo.firmwareRev), false);
            decodeControl1.AddText(string.Format("Firmware Part: {0}\r\n", verInfo.firmwarePartNumber), false);
            decodeControl1.AddText(string.Format("BootCode Rev: {0}\r\n", verInfo.bootCodeRev), false);
            decodeControl1.AddText(string.Format("Decoder Rev: {0}\r\n", verInfo.decoderRev), false);
            decodeControl1.AddText(string.Format("Scan Driver Rev: {0}\r\n", verInfo.scanDriverRev), false);
            decodeControl1.AddText(string.Format("Device Type: {0}\r\n", verInfo.deviceType), false);
            decodeControl1.AddText(string.Format("Manufacturer ID: {0}\r\n\r\n", verInfo.manufacturersId), true);
        }
        /// <summary>
        /// Method where connection to the imager is established.
        /// </summary>
        /// <returns></returns>
        private bool ConnectToImager()
        {
            bool bRetVal = true;
            // Display a wait cursor during connect.
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                // Connect at current port (stored in the DecodeControl). This will throw an exception if
                // connect() fails.
                decodeControl1.Connect();
                // Display the connected message.
                if (bIsSerialPort)
                    this.StatusLabel.Text = string.Format(" Imager Connected to {0} at {1} baud.", decodeControl1.commPort.ToString(), decodeControl1.serialPortCfg.BaudRate.ToString());
                else
                    this.StatusLabel.Text = string.Format(" Imager Connected to {0} (USB)", decodeControl1.commPort.ToString());
                // Sound the success beep.
                decodeControl1.PlaySounds(true);
            }
            catch (ImagerException ex)
            {
                // Connect failed.
                bRetVal = false;
                this.StatusLabel.Text = string.Format("Connect Failed: {0}", ex.Message);
                decodeControl1.PlaySounds(false);
            }
            finally
            {
                // Make sure to change the cursor back to the default.
                Cursor.Current = Cursors.Default;
            }
            return (bRetVal);
        }
        /// <summary>
        /// If exit selected, call Form close.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnExit(object sender, EventArgs e)
        {
            Close();
        }
        /// <summary>
        /// Recyle the current connection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnRecycleConnection(object sender, EventArgs e)
        {
            // If currently connect, disconnect.
            if (decodeControl1.IsConnected())
            {
                // Display disconnecting message.
                StatusLabel.Text = "Disconnecting";
                StatusLabel.Refresh();
                // disconnect from the imager.
                try
                {
                    decodeControl1.Disconnect();
                }
                catch (ImagerException ex)
                {
                    // Pretty much ignore any exceptions in case the problem is a disconnected imager.
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
            }
            // Start the reconnect timer.
            StatusLabel.Text = "Reconnecting Imager...";
            connectTimer.Enabled = true;
        }
        /// <summary>
        /// Display the about box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnAbout(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        /// <summary>
        /// Activate the SelectCommPort dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSelectCommPort(object sender, EventArgs e)
        {
            // Center the dialog over the application.
            selectCommPortDialog1.Location = new Point((Location.X + ((Width - selectCommPortDialog1.DialogWidth) / 2)), (Location.Y + ((Height - selectCommPortDialog1.DialogHeight) / 2)));
            // Show the dialog. It will return OK if the apply button is selected.
            if (selectCommPortDialog1.ShowDialog() == DialogResult.OK)
            {
                // If this is a new port, prepare to disconnect from current then reconnect.
                if (decodeControl1.commPort != selectCommPortDialog1.commPort)
                {
                    // Set the new port
                    decodeControl1.commPort = selectCommPortDialog1.commPort;
                    // Check to see if port is a serial port
                    bIsSerialPort = (string.Compare(selectCommPortDialog1.currentPortType, "Serial") == 0) ? true : false;
                    // Recycle the connection to disconnect from the current port and connect to the new port.
                    OnRecycleConnection(this, new EventArgs());
                }
            }
        }
        /// <summary>
        /// Display the configure symbologies dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void symbologyConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Set the symbology dialog location so it's centered on this form.
            symbologyDialog1.Location = new Point((Location.X + ((Width - symbologyDialog1.DialogWidth) / 2)), (Location.Y + ((Height - symbologyDialog1.DialogHeight) / 2)));
            // Run the dialog.
            symbologyDialog1.ShowDialog();
        }
    }
}
