#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Intelligent Image Sample (Assembly)
*
*  Description: Contains code to demonstrate how to capture intelligent image
*               data using the ImagerSdk assembly.  It also uses the
*               SymbologyDialog and SelectCommPortDialog components which are
*               included with the .NET SDK. 
* 
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
* 
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.CommPortComponent;
using Honeywell.WinForms.HHI.SymbologyComponent;

#endregion

namespace IQImageSample_Assembly_
{
    public partial class IQImageSample : Form
    {
        #region Private Members and Constants
        /// <summary>Aztec demo sheet values.</summary>
        private const int aztecAspectRatio = 1;
        private const int aztecOffsetX = 2;
        private const int aztecOffsetY = 42;
        private const int aztecWidth = 124;
        private const int aztecHeight = 58;
        /// <summary>Code 39 demo sheet values.</summary>
        private const int code39AspectRatio = 33;
        private const int code39OffsetX = -2;
        private const int code39OffsetY = 57;
        private const int code39Width = 180;
        private const int code39Height = 58;
        /// <summary>Matrix SDK instance.</summary>
        ImagerSdk sdk = new ImagerSdk();
        /// <summary>Intelligent Imaging Settings class instance.</summary>
        IQImageData iqSettings = new IQImageData();
        /// <summary>IQCapture in progress</summary>
        bool bCaptureInProgress = false;
        /// <summary>Capture initiated by hotkey?</summary>
        bool bHotKey = false;
        /// <summary>Is connect actually a reconnect?</summary>
        bool bReconnect = false;
        // Have we tried to set the comm port at least once.
        private bool bCommPortSet = false;
        /// <summary>End Private Members and Constants region</summary>
        #endregion

        #region Constructor

        /// <summary>IQImageSample constructor. No parameters.</summary>
        public IQImageSample()
        {
            InitializeComponent();
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // Hook IQImage and IQBarcode events
                sdk.iqBarcodeEvent += new ImagerSdk.IQBarcodeEventHandler(OnIqBarcodeEvent);
                sdk.iqImageEvent += new ImagerSdk.IQImageEventHandler(OnIqImageEvent);
                sdk.RegisterGlobalHotKey(Keys.Up, HotKeyModifiers.MOD_NONE);
                sdk.globalHotKeyEvent += new ImagerSdk.HotKeyEventHandler(OnHotKey);
                // for purposes of this sample we will force image to fit in our picture box. You
                // can set the maxWidth, maxHeight values larger if you want a larger image.
                iqSettings.maxWidth = pictureBoxIqImage.Width;
                iqSettings.maxHeight = pictureBoxIqImage.Height;
                // We'll start with the Aztec Demo
                buttonAztecDemo_Click(this, new EventArgs());
                // Display values.
                SetControlIqValues();
                // Set button states
                SetButtonEnables();
                // Refresh the display
                Refresh();
                // Set focus on the capture button
                buttonCaptureIQImage.Focus();
            }
        }
        // End Constructor
        #endregion

        #region Private Methods

        #region Private Non-Event Methods
        /// <summary>Private method that actually handles connecting to the imager</summary>
        private void ConnectToImager()
        {
            // Attempt to connect.
            try
            {
                // Set the connecting/reconnecting message in the status bar.
                labelStatus.Text = (bReconnect) ? "Reconnecting To The Imager..." : "Connecting To The Imager...";
                Refresh();
                // Call the sdk connect method (commPort already set so we use parameterless call.
                sdk.Connect();
                // Since no exceptions, set the connected message in the status bar.
                labelStatus.Text = "Connected On " + sdk.commPort.ToString();
            }
            catch (ImagerException ex)
            {
                // Display connect failure message
                labelStatus.Text = ((bReconnect) ? "Reconnect " : "Connect") + "Failed: " + ex.Message;
            }
            finally
            {
                // Set focus to the capture IQ Image button.
                buttonCaptureIQImage.Focus();
            }
        }
        /// <summary>Method that sets the IQ settings controls from the iqSettings class</summary>
        private void SetControlIqValues()
        {
            numericUpDownAspectRatio.Value = Convert.ToDecimal(iqSettings.aspectRatio);
            numericUpDownOffsetX.Value = Convert.ToDecimal(iqSettings.xOffset);
            numericUpDownOffsetY.Value = Convert.ToDecimal(iqSettings.yOffset);
            numericUpDownWidth.Value = Convert.ToDecimal(iqSettings.width);
            numericUpDownHeight.Value = Convert.ToDecimal(iqSettings.height);
            radioButtonBinary.Checked = iqSettings.sendBinary;
        }
        /// <summary>Method that Enables/Disables the setting preset buttons.</summary>
        private void SetButtonEnables()
        {
            // Aztec Demo Button
            buttonAztecDemo.Enabled = ((Convert.ToInt32(numericUpDownAspectRatio.Value) != aztecAspectRatio) ||
                                       (Convert.ToInt32(numericUpDownOffsetX.Value) != aztecOffsetX) ||
                                       (Convert.ToInt32(numericUpDownOffsetY.Value) != aztecOffsetY) ||
                                       (Convert.ToInt32(numericUpDownWidth.Value) != aztecWidth) ||
                                       (Convert.ToInt32(numericUpDownHeight.Value) != aztecHeight)) ? true : false;
            // Code 39 Button
            buttonCode39Demo.Enabled = ((Convert.ToInt32(numericUpDownAspectRatio.Value) != code39AspectRatio) ||
                                        (Convert.ToInt32(numericUpDownOffsetX.Value) != code39OffsetX) ||
                                        (Convert.ToInt32(numericUpDownOffsetY.Value) != code39OffsetY) ||
                                        (Convert.ToInt32(numericUpDownWidth.Value) != code39Width) ||
                                        (Convert.ToInt32(numericUpDownHeight.Value) != code39Height)) ? true : false;
        }
        #endregion

        #region Event Handlers
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void IQImageSample_Activated(object sender, EventArgs e)
        {
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                if (!bCommPortSet)
                {
                    bCommPortSet = true;
                    // Read the default Port
                    CommPorts commPort = new CommPorts();
                    sdk.commPort = commPort.DefaultCommPort;
                    // Load the current port and intialize the filter list into the select comm port dialog.
                    selectCommPortDialog1.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                    selectCommPortDialog1.commPort = sdk.commPort;
                    // If no default port defined, get one from the user.
                    if (selectCommPortDialog1.commPort == eCommPort.LAST_COM_PORT)
                    {
                        Refresh();
                        buttonComPort_Click(this, new EventArgs());
                    }
                    // If one is defined now, connect.
                    if (selectCommPortDialog1.commPort != eCommPort.LAST_COM_PORT)
                    {
                        bReconnect = false;
                        ConnectTimer.Interval = 100;
                        ConnectTimer.Enabled = true;
                    }
                    // We're done with the commPorts class so we can release the memory.
                    commPort.Dispose();
                }
            }
        }
        /// <summary>
        /// Called by SDK when an Intelligent Image Barcode is received.
        /// </summary>
        /// <param name="obj">Current instance of the Matrix Net SDK</param>
        /// <param name="iqBarcode">Barcode Event class which contains the event results, event type and barcode info.</param>
        private void OnIqBarcodeEvent(object obj, CBarcodeEvent iqBarcode)
        {
            // Reset the IQ picture
            pictureBoxIqImage.Image = null;
            // If the call was successful, we have an IQ barcode.
            if (iqBarcode.nResult == ImagerResult.RESULT_SUCCESS)
            {
                // Display successful status message.
                labelStatus.Text = "Received Intelligent Image Barcode.";
                // Display captured barcode.
                textBoxIqBarcode.Text = sdk.TranslateByteArray(iqBarcode.BarcodeData.rawMessage, iqBarcode.BarcodeData.length, true, true, false);
            }
            else
            {
                // Call failed. Clear out text.
                textBoxIqBarcode.Text = "";
                // Display failure message.
                labelStatus.Text = "Capture Intelligent Image Failed: " + sdk.imgrCaps.GetErrorString(iqBarcode.nResult);
                // Reset capture in progress flag.
                bCaptureInProgress = false;
                // Play failure sound.
                sdk.PlaySounds(false);
            }
        }
        /// <summary>
        /// Called by SDK when an Intelligent Image Image is received.
        /// </summary>
        /// <param name="obj">Current instance of the Matrix Net SDK</param>
        /// <param name="iqImage">Image  Event class which contains the event results, event type and barcode info.</param>
        private void OnIqImageEvent(object obj, CImageEvent iqImage)
        {
            // If the call was successful, we have an IQ image.
            if (iqImage.nResult == ImagerResult.RESULT_SUCCESS)
            {
                // Display successful status message.
                labelStatus.Text = "Intelligent Image Capture Successful";
                // Display iq image.
                pictureBoxIqImage.Image = ImageFromBuffer.GetImageFromBuffer(iqImage.Image.imgWidth, iqImage.Image.imgHeight, iqImage.Image.buffer);
                // Play success sound.
                sdk.PlaySounds(true);
            }
            else
            {
                // Clear out image.
                pictureBoxIqImage.Image = null;
                // Display failure message.
                labelStatus.Text = "Capture Intelligent Image Failed: " + sdk.imgrCaps.GetErrorString(iqImage.nResult);
                // Play failure sound.
                sdk.PlaySounds(false);
            }
            // Reset capture in progress flag.
            bCaptureInProgress = false;
        }
        /// <summary>
        /// Called when the registered global hotkey is pressed.
        /// </summary>
        /// <param name="obj">The current instance of the Matrix SDK</param>
        /// <param name="hotKey">HotKey event class which contains hotkey data.</param>
        private void OnHotKey(object obj, CHotKey hotKey)
        {
            // If not already capturing an intelligent image, and hotkey not still down and
            // it's really our hotkey (which is should be but it's always a good idea to check).
            if (!bCaptureInProgress && !bHotKey && (((Keys)hotKey.HotKey) == Keys.Up) )
            {
                // Set status to capturing Intelligent Image
                labelStatus.Text = "Capturing Intelligent Image...";
                Refresh();
                // Attempt to capture an iq image.
                try
                {
                    // Set capture flag
                    bCaptureInProgress = true;
                    // Capture is due to hotkey.
                    bHotKey = true;
                    // Capture intelligent image with a 10 second decode timeout.
                    sdk.CaptureIntelligentImageNoWait(iqSettings, 10000);
                }
                catch (ImagerException ex)
                {
                    // Capture failed so show failure message.
                    labelStatus.Text = "Capture Intelligent Image Failed: " + ex.Message;
                    // Reset capture in progress flag.
                    bCaptureInProgress = false;
                    // Play failure sound.
                    sdk.PlaySounds(false);
                }
            }
        }

        #endregion

        #endregion

        #region Designer Generated Private Methods

        #region Miscellaneous
        /// <summary>
        /// We use a timer to allow the GUI to finish redrawing before starting a connect/reconnect.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConnectTimer_Tick(object sender, EventArgs e)
        {
            // Stop timer 
            ConnectTimer.Enabled = false;
            // Call connect.
            try
            {
                ConnectToImager();
            }
            catch (ImagerException ex)
            {
                this.labelStatus.Text = "Connect Failed: " + ex.Message;
            }
        }
        /// <summary>
        /// Look for the hotkey up message since the system only sends the WM_HOTKEY message when
        /// a hotkey is pressed, not released and we want the key to work like a trigger.
        /// 
        /// ***NOTE*** You MUST set the form's "KeyPreview" property (under Misc) to true for this to work.
        /// 
        /// </summary>
        /// <param name="obj">The current instance of the Matrix SDK</param>
        /// <param name="e">Key being released</param>
        private void IQImageSample_KeyUp(object sender, KeyEventArgs e)
        {
            // If it's the hotkey being released 
            if (e.KeyData == Keys.Up)
            {
                // If a capture is in progress, cancel the current operation
                if (bCaptureInProgress)
                    sdk.CancelIO();
                // Reset the hotkey down flag.
                bHotKey = false;
            }
        }
        #endregion

        #region Menu Items
        /// <summary>
        /// File-->Exit menu item selected. Close the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Help-->About menu item selected. Display the about box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        #endregion

        #region IQImage Settings
        /// <summary>
        /// Aspect ratio value changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownAspectRatio_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.aspectRatio = Convert.ToInt32(numericUpDownAspectRatio.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// X Offset to image value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownOffsetX_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.xOffset = Convert.ToInt32(numericUpDownOffsetX.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Y Offset to image value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownOffsetY_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.yOffset = Convert.ToInt32(numericUpDownOffsetY.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Image width value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownWidth_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.width = Convert.ToInt32(numericUpDownWidth.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Image height value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownHeight_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.height = Convert.ToInt32(numericUpDownHeight.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Transfer and display image as black and white.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonBinary_Click(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.sendBinary = true;
        }
        /// <summary>
        /// Transfer and display image in 8 bit/pixel grayscale.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonGrayScale_Click(object sender, EventArgs e)
        {
            // Save setting.
            iqSettings.sendBinary = false;
        }
        /// <summary>
        /// Set settings to match Aztec IQ Package Demo sheet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAztecDemo_Click(object sender, EventArgs e)
        {
            iqSettings.aspectRatio = aztecAspectRatio;
            iqSettings.xOffset = aztecOffsetX;
            iqSettings.yOffset = aztecOffsetY;
            iqSettings.width = aztecWidth;
            iqSettings.height = aztecHeight;

            SetControlIqValues();
        }
        /// <summary>
        /// Set settings to match Code39 IQ Demo sheet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCode39Demo_Click(object sender, EventArgs e)
        {
            iqSettings.aspectRatio = code39AspectRatio;
            iqSettings.xOffset = code39OffsetX;
            iqSettings.yOffset = code39OffsetY;
            iqSettings.width = code39Width;
            iqSettings.height = code39Height;

            SetControlIqValues();
        }
        #endregion

        #region Button Events
        /// <summary>
        /// Display the configure symbology dialog (Honeywell.WinForms.HHI.SymbologyComponent.SymbologyDialog).
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSymbologies_Click(object sender, EventArgs e)
        {
            symbologyDialog1.ShowDialog();
        }
        /// <summary>
        /// Display the select commPort dialog (Honeywell.WinForms.HHI.SymbologyComponent.SelectCommPortDialog).
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonComPort_Click(object sender, EventArgs e)
        {
            // Intialize the commPort dialog commPort.
            selectCommPortDialog1.commPort = sdk.commPort;
            // Show the dialog. It will return OK if the user selected a new commPort.
            if (selectCommPortDialog1.ShowDialog() == DialogResult.OK)
            {
                // Save the commPort dialog commPort to the sdk.
                sdk.commPort = selectCommPortDialog1.commPort;
                // Refresh the screen to finish getting rid of the commPort dialog.
                Refresh();
                // If the imager is currently connected.
                if (sdk.IsConnected())
                {
                    // Disconnect from the imager. Always do SDK calls in a try block since they throw ImagerExceptions
                    // if the call fails.
                    try
                    {
                        // Update the status message to disconnecting and force an app gui refresh.
                        labelStatus.Text = "Disconnecting...";
                        Refresh();
                        // Disconnect.
                        sdk.Disconnect();
                        // Wait 1/2 second for the port to fully close.
                        System.Threading.Thread.Sleep(500);
                        // Set the reconnecting flag and start the connect timer.
                        bReconnect = true;
                        ConnectTimer.Enabled = true;
                    }
                    catch (ImagerException ex)
                    {
                        // Disconnect failed. Display failure message.
                        labelStatus.Text = "Reconnect Failed: " + ex.Message;
                        // Play failure sound.
                        sdk.PlaySounds(false);
                    }
                }
                else
                {
                    // Set the reconnecting flag and start the connect timer.
                    bReconnect = true;
                    ConnectTimer.Enabled = true;
                }
            }
        }
        /// <summary>
        /// Called when Capture IQImage button selected. Calls non-blocking the SDK method to capture an intelligent image.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCaptureIQImage_Click(object sender, EventArgs e)
        {
            // Prevent multiple calls by checking the in progress flag.
            if (!bCaptureInProgress)
            {
                // Set the status to capturing.
                labelStatus.Text = "Capturing Intelligent Image...";
                Refresh();
                // Attempt to capture IQ image
                try
                {
                    // Reset the IQ picture
                    pictureBoxIqImage.Image = null;
                    // Set the capture in progress flag
                    bCaptureInProgress = true;
                    // Call the sdk method for non-blocking intelligent image capture (10 second decode timeout).
                    sdk.CaptureIntelligentImageNoWait(iqSettings, 10000);
                }
                catch (ImagerException ex)
                {
                    // Call failed, display message.
                    labelStatus.Text = "Capture Intelligent Image Failed: " + ex.Message;
                    // Reset capture in progress flag
                    bCaptureInProgress = false;
                    // Play failure sound.
                    sdk.PlaySounds(false);
                }
            }
        }
        #endregion 

        #endregion

    }
}
