#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Intelligent Image Sample (Control)
*
*  Description: Contains code to demonstrate how to capture intelligent image
*               data using the Matrix DecodeControl.  It also uses the
*               SymbologyDialog and SelectCommPortDialog components which are
*               included with the .NET SDK.
*
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.CommPortComponent;

#endregion

namespace IQImageSample_Control_
{
    public partial class IQImageSample : Form
    {
        #region Private Constants
        /// <summary>Aztec demo sheet values.</summary>
        private const int aztecAspectRatio = 1;
        private const int aztecOffsetX = 2;
        private const int aztecOffsetY = 42;
        private const int aztecWidth = 124;
        private const int aztecHeight = 58;
        /// <summary>Code 39 demo sheet values.</summary>
        private const int code39AspectRatio = 33;
        private const int code39OffsetX = -2;
        private const int code39OffsetY = 57;
        private const int code39Width = 180;
        private const int code39Height = 58;
        /// <summary>IQCapture in progress</summary>
        bool bCaptureInProgress = false;
        /// <summary>Is connect actually a reconnect?</summary>
        bool bReconnect = false;
        // Have we tried to set the comm port at least once.
        private bool bCommPortSet = false;
        /// <summary>End Private Members and Constants region</summary>
        #endregion

        #region Constructor

        /// <summary>IQImageSample constructor. No parameters.</summary>
        public IQImageSample()
        {
            InitializeComponent();
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // for purposes of this sample we will force image to fit in our picture box. You
                // can set the maxWidth, maxHeight values larger if you want a larger image.
                decodeControlIqBarcode.IQImageSettings.maxWidth = pictureBoxIqImage.Width;
                decodeControlIqBarcode.IQImageSettings.maxHeight = pictureBoxIqImage.Height;
                // We'll start with the Aztec Demo
                buttonAztecDemo_Click(this, new EventArgs());
                // Display values.
                SetControlIqValues();
                // Set button states
                SetButtonEnables();
                // Refresh the display
                Refresh();
                // Set focus on the decode control
                decodeControlIqBarcode.Focus();
            }
        }
        // End Constructor
        #endregion

        #region Private Methods

        #region Private Non-Event Methods
        /// <summary>Private method that actually handles connecting to the imager</summary>
        private void ConnectToImager()
        {
            // Attempt to connect.
            try
            {
                // Set the connecting/reconnecting message in the status bar.
                labelStatus.Text = (bReconnect) ? "Reconnecting To The Imager..." : "Connecting To The Imager...";
                Refresh();
                // Call the sdk connect method (commPort already set so we use parameterless call.
                decodeControlIqBarcode.Connect();
                // Since no exceptions, set the connected message in the status bar.
                labelStatus.Text = "Connected On " + decodeControlIqBarcode.commPort.ToString();
            }
            catch (ImagerException ex)
            {
                // Display connect failure message
                labelStatus.Text = ((bReconnect) ? "Reconnect " : "Connect") + "Failed: " + ex.Message;
            }
            finally
            {
                // Set focus on the decode control.
                decodeControlIqBarcode.Focus();
            }
        }
        /// <summary>Method that sets the IQ settings controls from the iqSettings class</summary>
        private void SetControlIqValues()
        {
            numericUpDownAspectRatio.Value = Convert.ToDecimal(decodeControlIqBarcode.IQImageSettings.aspectRatio);
            numericUpDownOffsetX.Value = Convert.ToDecimal(decodeControlIqBarcode.IQImageSettings.xOffset);
            numericUpDownOffsetY.Value = Convert.ToDecimal(decodeControlIqBarcode.IQImageSettings.yOffset);
            numericUpDownWidth.Value = Convert.ToDecimal(decodeControlIqBarcode.IQImageSettings.width);
            numericUpDownHeight.Value = Convert.ToDecimal(decodeControlIqBarcode.IQImageSettings.height);
            radioButtonBinary.Checked = decodeControlIqBarcode.IQImageSettings.sendBinary;
        }
        /// <summary>Method that Enables/Disables the setting preset buttons.</summary>
        private void SetButtonEnables()
        {
            // Aztec Demo Button
            buttonAztecDemo.Enabled = ((Convert.ToInt32(numericUpDownAspectRatio.Value) != aztecAspectRatio) ||
                                       (Convert.ToInt32(numericUpDownOffsetX.Value) != aztecOffsetX) ||
                                       (Convert.ToInt32(numericUpDownOffsetY.Value) != aztecOffsetY) ||
                                       (Convert.ToInt32(numericUpDownWidth.Value) != aztecWidth) ||
                                       (Convert.ToInt32(numericUpDownHeight.Value) != aztecHeight)) ? true : false;
            // Code 39 Button
            buttonCode39Demo.Enabled = ((Convert.ToInt32(numericUpDownAspectRatio.Value) != code39AspectRatio) ||
                                        (Convert.ToInt32(numericUpDownOffsetX.Value) != code39OffsetX) ||
                                        (Convert.ToInt32(numericUpDownOffsetY.Value) != code39OffsetY) ||
                                        (Convert.ToInt32(numericUpDownWidth.Value) != code39Width) ||
                                        (Convert.ToInt32(numericUpDownHeight.Value) != code39Height)) ? true : false;
        }
        #endregion

        #region Event Handlers
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void IQImageSample_Activated(object sender, EventArgs e)
        {
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                if (!bCommPortSet)
                {
                    bCommPortSet = true;
                    // Read the default Port
                    CommPorts commPort = new CommPorts();
                    decodeControlIqBarcode.commPort = commPort.DefaultCommPort;
                    // Load the current port and intialize the filter list into the select comm port dialog.
                    selectCommPortDialog1.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                    selectCommPortDialog1.commPort = decodeControlIqBarcode.commPort;
                    // If no default port defined, get one from the user.
                    if (selectCommPortDialog1.commPort == eCommPort.LAST_COM_PORT)
                    {
                        Refresh();
                        buttonComPort_Click(this, new EventArgs());
                    }
                    // If one is defined now, connect.
                    if (selectCommPortDialog1.commPort != eCommPort.LAST_COM_PORT)
                    {
                        bReconnect = false;
                        ConnectTimer.Interval = 100;
                        ConnectTimer.Enabled = true;
                    }
                    // We're done with the commPorts class so we can release the memory.
                    commPort.Dispose();
                }
            }
        }
        /// <summary>
        /// Called by SDK when an imager event is received. These include intelligent image barcode and image.
        /// </summary>
        /// <param name="obj">Current instance of the Matrix Net SDK</param>
        /// <param name="iqBarcode">Barcode Event class which contains the event results, event type and barcode info.</param>
        private void decodeControlIqBarcode_DataEvent(object theDecodeControl, DataEvent dataEvent)
        {
            // Is this an Intelligent image event?
            if (dataEvent.EventType == EventType.ET_IQ_IMAGE)
            {
                // If the call was successful, we have an IQ image.
                if (dataEvent.nResult == ImagerResult.RESULT_SUCCESS)
                {
                    // Display iq image.
                    pictureBoxIqImage.Image = decodeControlIqBarcode.IQImage;
                    // Display successful status message.
                    labelStatus.Text = "Intelligent Image Capture Successful";
                    // Play success sound.
                    if( !decodeControlIqBarcode.AutoSounds )
                        decodeControlIqBarcode.PlaySounds(true);
                }
                else
                {
                    // Clear out image.
                    pictureBoxIqImage.Image = null;
                    // Display failure message.
                    labelStatus.Text = decodeControlIqBarcode.imgrCaps.GetErrorString(dataEvent.nResult);
                    // Play failure sound.
                    if (!decodeControlIqBarcode.AutoSounds)
                        decodeControlIqBarcode.PlaySounds(false);
                }
                // Reset capture in progress flag.
                bCaptureInProgress = false;
            }
            else if (dataEvent.EventType != EventType.ET_IQ_BARCODE)
            {
                // Clear out image.
                pictureBoxIqImage.Image = null;
                // We got a standard barcode instead.
                if (dataEvent.EventType == EventType.ET_BARCODE)
                    labelStatus.Text = "Received Non-Intelligent Image Barcode.";
                else if( dataEvent.EventType == EventType.ET_TEXT_MSG )
                    labelStatus.Text = decodeControlIqBarcode.TextMsgData.message;
                else
                    labelStatus.Text = string.Format( "Received Text Message. {0}",dataEvent.EventType.ToString() );
                // Reset capture in progress flag.
                bCaptureInProgress = false;
            }
            else
            {
                if( dataEvent.nResult == ImagerResult.RESULT_SUCCESS )
                {
                    // Display successful status message.
                    labelStatus.Text = "Received Intelligent Image Barcode.";
                }
                else
                {
                    // Call failed. Clear out text.
                    decodeControlIqBarcode.Text = "";
                    // Display failure message.
                    labelStatus.Text = "Capture Intelligent Image Failed: " + decodeControlIqBarcode.imgrCaps.GetErrorString(dataEvent.nResult);
                    // Reset capture in progress flag.
                    bCaptureInProgress = false;
                    // Play failure sound.
                    decodeControlIqBarcode.PlaySounds(false);
                }
            }
        }
        #endregion

        #endregion

        #region Designer Generated Private Methods

        #region Miscellaneous
        /// <summary>
        /// We use a timer to allow the GUI to finish redrawing before starting a connect/reconnect.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConnectTimer_Tick(object sender, EventArgs e)
        {
            // Stop timer
            ConnectTimer.Enabled = false;
            // Call connect.
            try
            {
                ConnectToImager();
            }
            catch (ImagerException ex)
            {
                this.labelStatus.Text = "Connect Failed: " + ex.Message;
            }
        }
        #endregion

        #region Menu Items
        /// <summary>
        /// File-->Exit menu item selected. Close the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            decodeControlIqBarcode.Disconnect();
            this.Close();
        }
        /// <summary>
        /// Help-->About menu item selected. Display the about box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        #endregion

        #region IQImage Settings
        /// <summary>
        /// Aspect ratio value changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownAspectRatio_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.aspectRatio = Convert.ToInt32(numericUpDownAspectRatio.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// X Offset to image value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownOffsetX_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.xOffset = Convert.ToInt32(numericUpDownOffsetX.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Y Offset to image value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownOffsetY_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.yOffset = Convert.ToInt32(numericUpDownOffsetY.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Image width value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownWidth_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.width = Convert.ToInt32(numericUpDownWidth.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Image height value changed. Note: value is in terms of "minimum bar widths".
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownHeight_ValueChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.height = Convert.ToInt32(numericUpDownHeight.Value);
            // Update preset button enable states.
            SetButtonEnables();
        }
        /// <summary>
        /// Transfer and display image as black and white.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonBinary_Click(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.sendBinary = true;
        }
        /// <summary>
        /// Set settings to match Aztec IQ Package Demo sheet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioButtonGrayScale_CheckedChanged(object sender, EventArgs e)
        {
            // Save setting.
            decodeControlIqBarcode.IQImageSettings.sendBinary = false;
        }
        /// <summary>
        /// Set settings to match Aztec IQ Package Demo sheet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAztecDemo_Click(object sender, EventArgs e)
        {
            decodeControlIqBarcode.IQImageSettings.aspectRatio = aztecAspectRatio;
            decodeControlIqBarcode.IQImageSettings.xOffset = aztecOffsetX;
            decodeControlIqBarcode.IQImageSettings.yOffset = aztecOffsetY;
            decodeControlIqBarcode.IQImageSettings.width = aztecWidth;
            decodeControlIqBarcode.IQImageSettings.height = aztecHeight;

            SetControlIqValues();
        }
        /// <summary>
        /// Set settings to match Code39 IQ Demo sheet.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCode39Demo_Click(object sender, EventArgs e)
        {
            decodeControlIqBarcode.IQImageSettings.aspectRatio = code39AspectRatio;
            decodeControlIqBarcode.IQImageSettings.xOffset = code39OffsetX;
            decodeControlIqBarcode.IQImageSettings.yOffset = code39OffsetY;
            decodeControlIqBarcode.IQImageSettings.width = code39Width;
            decodeControlIqBarcode.IQImageSettings.height = code39Height;

            SetControlIqValues();
        }

        #endregion

        #region Button Events
        /// <summary>
        /// Display the configure symbology dialog (Honeywell.WinForms.HHI.SymbologyComponent.SymbologyDialog).
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSymbologies_Click(object sender, EventArgs e)
        {
            symbologyDialog1.ShowDialog();
        }
        /// <summary>
        /// Display the select commPort dialog (Honeywell.WinForms.HHI.SymbologyComponent.SelectCommPortDialog).
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonComPort_Click(object sender, EventArgs e)
        {
            // Intialize the commPort dialog commPort.
            selectCommPortDialog1.commPort = decodeControlIqBarcode.commPort;
            // Show the dialog. It will return OK if the user selected a new commPort.
            if (selectCommPortDialog1.ShowDialog() == DialogResult.OK)
            {
                // Save the new commPort setting.
                decodeControlIqBarcode.commPort = selectCommPortDialog1.commPort;
                // Refresh the screen to finish getting rid of the commPort dialog.
                Refresh();
                if (decodeControlIqBarcode.IsConnected())
                {
                    // Disconnect from the imager. Always do SDK calls in a try block since they throw ImagerExceptions
                    // if the call fails.
                    try
                    {
                        // Update the status message to disconnecting and force an app gui refresh.
                        labelStatus.Text = "Disconnecting...";
                        labelStatus.Refresh();
                        // Disconnect.
                        decodeControlIqBarcode.Disconnect();
                        // Wait 1/2 second for the port to fully close.
                        System.Threading.Thread.Sleep(500);
                        // Set the reconnecting flag and start the connect timer.
                        bReconnect = true;
                        ConnectTimer.Enabled = true;
                    }
                    catch (ImagerException ex)
                    {
                        // Disconnect failed. Display failure message.
                        labelStatus.Text = "Reconnect Failed: " + ex.Message;
                        // Play failure sound.
                        decodeControlIqBarcode.PlaySounds(false);
                    }
                }
                else
                {
                    // Set the reconnecting flag and start the connect timer.
                    bReconnect = true;
                    ConnectTimer.Enabled = true;
                }
            }
        }
        /// <summary>
        /// Called when Capture IQImage button selected. Calls non-blocking the SDK method to capture an intelligent image.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCaptureIQImage_Click(object sender, EventArgs e)
        {
            // Prevent multiple calls by checking the in progress flag.
            if (!bCaptureInProgress)
            {
                // Set the status to capturing.
                labelStatus.Text = "Capturing Intelligent Image...";
                Refresh();
                // Attempt to capture IQ image
                try
                {
                    // Reset the IQ picture
                    pictureBoxIqImage.Image = null;
                    // Set the capture in progress flag
                    bCaptureInProgress = true;
                    // Call the control method for non-blocking intelligent image capture.
                    decodeControlIqBarcode.CaptureIntelligentImageNoWait();
                }
                catch (ImagerException ex)
                {
                    // Call failed, display message.
                    labelStatus.Text = "Capture Intelligent Image Failed: " + ex.Message;
                    // Reset capture in progress flag
                    bCaptureInProgress = false;
                    // Play failure sound.
                    decodeControlIqBarcode.PlaySounds(false);
                }
            }
        }
        #endregion

        #endregion
    }
}
