#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Image Capture Sample (Assembly)
*
*  Description: Contains code to demonstrate how to capture an image using the
*               MatrixNetSdk assembly.  It also uses the SelectCommPortDialog
*               components which is included with the .NET SDK.
*
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using Honeywell.WinForms.HHI.Utility;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.CommPortComponent;

#endregion

namespace ImageSample_Assembly_
{
    public partial class ImageSample : Form
    {
        #region Constructor
        /// <summary>Image Sample using the MatrixNetSdk and a picturebox control constructor.</summary>
        public ImageSample()
        {
            InitializeComponent();
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // Hook the image event, and image transfer event.
                sdk.imageEvent += new ImagerSdk.ImageEventHandler(OnImageReceived);
                sdk.transferEvent += new ImagerSdk.TransferProgressHandler(OnImageTransferUpdateReceived);
                sdk.commPort = eCommPort.LAST_COM_PORT;
            }
            comboBoxDisplayMode.SelectedIndex = (pictureBoxImage.SizeMode == PictureBoxSizeMode.Normal) ? 0 : 1;
            checkBoxAutoCrop.Checked = sdk.imgTrans.AutoCrop;
        }

        #endregion

        #region Private Members
        /// <summary>Matrix SDK class instance used to communicate with the imager.</summary>
        ImagerSdk sdk = new ImagerSdk();
        /// <summary>CommPorts class instance used to determine whether port is a serial port or not</summary>
        private CommPorts commPorts = new CommPorts();
        /// <summary>Is current port a serial port?.</summary>
        private bool bIsSerialPort = false;
        /// <summary>Image capture in progress?.</summary>
        private bool bImageCapture = false;
        /// <summary>Trigger mode on the way in.</summary>
        private TriggerMode triggerMode = TriggerMode.TM_MANUAL_SERIAL;

        #endregion

        #region Private Methods

        /// <summary>
        /// Method where connection to the imager is established.
        /// </summary>
        /// <returns></returns>
        private bool ConnectToImager()
        {
            bool bRetVal = true;
            // Display a wait cursor during connect.
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                // Set status to connecting.
                labelStatusBox.Text = "Connecting To The Imager...";
                Refresh();
                // Connect at current port (stored in the DecodeControl). This will throw an exception if
                // connect() fails.
                sdk.Connect();
                // Check the current trigger mode
                if (sdk.triggerCfg.mode != TriggerMode.TM_SNAP_AND_SHIP)
                    triggerMode = sdk.triggerCfg.mode;
                // Set to snap & ship trigger mode.
                SetTriggerMode(true);
                // Display the connected message.
                if (bIsSerialPort)
                    labelStatusBox.Text = string.Format(" Imager Connected to {0} at {1} baud.", sdk.commPort.ToString(), sdk.serialPortCfg.BaudRate.ToString());
                else
                    labelStatusBox.Text = string.Format(" Imager Connected to {0} (USB)", sdk.commPort.ToString());
                // Sound the success beep.
                sdk.PlaySounds(true);
            }
            catch (ImagerException ex)
            {
                // Connect failed.
                bRetVal = false;
                labelStatusBox.Text = string.Format("Connect Failed: {0}", ex.Message);
                sdk.PlaySounds(false);
            }
            finally
            {
                // Make sure to change the cursor back to the default.
                Cursor.Current = Cursors.Default;
            }
            return (bRetVal);
        }
        /// <summary>
        /// Set the imager trigger mode to snap and ship or reset to barcode capture.
        /// </summary>
        /// <param name="bSnapAndShip"></param>
        private void SetTriggerMode(bool bSnapAndShip)
        {
            // Set the trigger mode.
            sdk.triggerCfg.mode = (bSnapAndShip) ? TriggerMode.TM_SNAP_AND_SHIP : triggerMode;
            // Set the correct mask
            sdk.triggerCfg.mask = TriggerConfig.BitMask.TRIG_MODE;
            // Set the trigger mode.
            try
            {
                sdk.triggerCfg.WriteConfig(false);
            }
            catch (ImagerException ex)
            {
                labelStatusBox.Text = ex.Message;
            }
        }
        /// <summary>
        /// Called whenever an image is received from the imager. The image is stored in the Image member of
        /// the CImageEvent class imgEvent.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="imgEvent">Image event data as sent by the SDK. It contains the image data as well as the capture result code.</param>
        private void OnImageReceived(object obj, CImageEvent imgEvent)
        {
            // Reset the capturing flag
            bImageCapture = false;
            // Status message.
            string strTmp = "Image Capture Successful.";
            bool bResult = true;
            // If successful capture, display the image.
            if (imgEvent.nResult == ImagerResult.RESULT_SUCCESS)
                pictureBoxImage.Image = ImageFromBuffer.GetImageFromBuffer(imgEvent.Image.imgWidth, imgEvent.Image.imgHeight, imgEvent.Image.buffer);
            else
                strTmp = "Capture Image Failed: " + sdk.GetErrorMessage(imgEvent.nResult);

            sdk.PlaySounds(bResult);
            labelStatusBox.Text = strTmp;
        }
        /// <summary>
        /// Called when image transfer updates are sent from the SDK. The CTransferProgress instance transProg contains the transfer percent complete.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="transProg">CTransferProgress data for this call: Bytes to transfer, bytes sent so far, percent complete.</param>
        private void OnImageTransferUpdateReceived(object obj, CTransferProgress transProg)
        {
            labelStatusBox.Text = string.Format("Image {0}% Transferred", transProg.PercentComplete.ToString());
            Refresh();
        }

        #endregion

        #region Designer Generated Private Methods

        // Private member used to prevent re-entrance.
        private bool bPortDialogCalled = false;
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void ImageSample_Activated(object sender, EventArgs e)
        {
            if (bPortDialogCalled )
                return;
            bPortDialogCalled = true;
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // Set the default commPort
                sdk.commPort = commPorts.DefaultCommPort; ;
                // Load the current port and intialize the filter list into the select comm port dialog.
                selectPortDlg.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                selectPortDlg.commPort = sdk.commPort;
                // If no default port defined, get one from the user.
                if (sdk.commPort == eCommPort.LAST_COM_PORT)
                {
                    Refresh();
                    selectCOMPortToolStripMenuItem_Click(this, new EventArgs());
                }
                // If one is defined now, connect.
                if (sdk.commPort != eCommPort.LAST_COM_PORT)
                {
                    ConnectTimer.Interval = 100;
                    ConnectTimer.Enabled = true;
                }
                // We're done with the commPorts class so we can release the memory.
                commPorts.Dispose();
            }
        }
        /// <summary>
        /// If exit selected, call Form close.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Activate the SelectCommPort dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectCOMPortToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Show the dialog. It will return OK if the apply button is selected.
            if (selectPortDlg.ShowDialog() == DialogResult.OK)
            {
                // If this is a new port, prepare to disconnect from current then reconnect.
                if (sdk.commPort != selectPortDlg.commPort)
                {
                    // Set the new port
                    sdk.commPort = selectPortDlg.commPort;
                    // Check to see if port is a serial port
                    bIsSerialPort = (string.Compare(selectPortDlg.currentPortType, "Serial") == 0) ? true : false;
                    // Recycle the connection to disconnect from the current port and connect to the new port.
                    recycleConnectionToolStripMenuItem_Click(this, new EventArgs());
                }
            }
        }
        /// <summary>
        /// Recyle the current connection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void recycleConnectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // If currently connect, disconnect.
            if (sdk.IsConnected())
            {
                // Display disconnecting message.
                labelStatusBox.Text = "Disconnecting";
                Refresh();
                // disconnect from the imager.
                try
                {
                    sdk.Disconnect();
                }
                catch (ImagerException ex)
                {
                    // Pretty much ignore any exceptions in case the problem is a disconnected imager.
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
            }
            // Start the reconnect timer.
            labelStatusBox.Text = "Reconnecting Imager...";
            ConnectTimer.Enabled = true;
        }
        /// <summary>
        /// Display the about box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        /// <summary>
        /// Image capture button selected. Calls sdk non-blocking capture method.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonImageCapture_Click(object sender, EventArgs e)
        {
            if (!bImageCapture)
            {
                try
                {
                    // Update status to capturing
                    labelStatusBox.Text = "Capturing Image...";
                    Refresh();
                    ImageAcquisition imageAcqu = new ImageAcquisition();
                    ImageTransfer imageTrans = new ImageTransfer();
                    // Capture the image.
                    sdk.CaptureImageNoWait(imageAcqu, imageTrans);
                    // Set the capture in progress flag.
                    bImageCapture = true;
                }
                catch (ImagerException ex)
                {
                    sdk.PlaySounds(false);
                    labelStatusBox.Text = "Capture Image Failed: " + ex.Message;
                }
            }
        }
        /// <summary>
        /// Clears the picture box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClearView_Click(object sender, EventArgs e)
        {
            pictureBoxImage.Image = null;
        }
        /// <summary>
        /// The autocrop checkbox check state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBoxAutoCrop_Click(object sender, EventArgs e)
        {
            sdk.imgTrans.AutoCrop = checkBoxAutoCrop.Checked;
        }
        /// <summary>
        /// The user has selected a new image display mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBoxDisplayMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxImage.SizeMode = (comboBoxDisplayMode.SelectedIndex == 0) ? PictureBoxSizeMode.Normal : PictureBoxSizeMode.Zoom;
        }
        /// <summary>
        /// A timer is used to start connect so that the gui has time to redraw on app start or recycle.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        private void ConnectTimer_Tick(object sender, EventArgs e)
        {
            ConnectTimer.Enabled = false;
            try
            {
                ConnectToImager();
            }
            catch (ImagerException ex)
            {
                labelStatusBox.Text = ("Connect Failed" + ex.Message);
            }
        }

        #endregion

    }
}
