#region Documentation
/*
******************************************************************************
*
*             Copyright (c) 2007 Honeywell International, Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
*  Application: Image Capture Sample (Control)
*
*  Description: Contains code to demonstrate how to capture an image using the
*               Matrix ImageControl.  It also uses the SelectCommPortDialog
*               components which is included with the .NET SDK.
*
*   UNLESS OTHER WISE AGREED TO BY HAND HELD PRODUCTS AND THE USER OF THIS CODE,
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
*   EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
*   WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  04/10/2007   JCM   Original Release
*
******************************************************************************
*/

#endregion

#region Namespace References

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using Honeywell.WinForms.HHI.ImageControl;
using Honeywell.WinForms.HHI.SDK;
using Honeywell.WinForms.HHI.CommPortComponent;

#endregion

namespace ImageSample_Control_
{
    public partial class ImageSample : Form
    {
        #region Constructor
        /// <summary>Image Sample using an ImageControl constructor.</summary>
        public ImageSample()
        {
            InitializeComponent();

            comboBoxDisplayMode.SelectedIndex = (SampleImageControl.ImageDisplay == ImageControl.ImageDisplayMode.Normal) ? 0 : 1;
            checkBoxAutoCrop.Checked = SampleImageControl.imgTrans.AutoCrop;
        }

        #endregion

        #region Private Members
        /// <summary>CommPorts class instance used to determine whether port is a serial port or not</summary>
        private CommPorts commPorts = new CommPorts();
        /// <summary>Is current port a serial port?.</summary>
        private bool bIsSerialPort = false;
        /// <summary>Image capture in progress?.</summary>
        private bool bImageCapture = false;
        /// <summary>Trigger mode on the way in.</summary>
        private TriggerMode triggerMode = TriggerMode.TM_MANUAL_SERIAL;
        // Private member used to prevent re-entrance.
        private bool bPortDialogCalled = false;

        #endregion

        #region Private Methods
        /// <summary>
        /// Method where connection to the imager is established.
        /// </summary>
        /// <returns></returns>
        private bool ConnectToImager()
        {
            bool bRetVal = true;
            // Display a wait cursor during connect.
            Cursor.Current = Cursors.WaitCursor;
            try
            {
                // Connect at current port (stored in the DecodeControl). This will throw an exception if
                // connect() fails.
                SampleImageControl.Connect();
                // Check the current trigger mode
                if (SampleImageControl.triggerCfg.mode != TriggerMode.TM_SNAP_AND_SHIP)
                    triggerMode = SampleImageControl.triggerCfg.mode;
                // Display the connected message.
                if (bIsSerialPort)
                    this.StatusBox.Text = string.Format(" Imager Connected to {0} at {1} baud.", SampleImageControl.commPort.ToString(), SampleImageControl.serialPortCfg.BaudRate.ToString());
                else
                    this.StatusBox.Text = string.Format(" Imager Connected to {0} (USB)", SampleImageControl.commPort.ToString());
                // Sound the success beep.
                SampleImageControl.PlaySounds(true);
            }
            catch (ImagerException ex)
            {
                // Connect failed.
                bRetVal = false;
                this.StatusBox.Text = string.Format("Connect Failed: {0}", ex.Message);
                SampleImageControl.PlaySounds(false);
            }
            finally
            {
                // Make sure to change the cursor back to the default.
                Cursor.Current = Cursors.Default;
                SampleImageControl.Focus();
            }
            return (bRetVal);
        }
        #endregion

        #region Designer Generated Private Methods
        /// <summary>
        /// Called when form becomes active. This method will display the comm port dialog if
        /// no comm port has been selected yet.
        /// </summary>
        /// <param name="sender">Usused</param>
        /// <param name="e">Unused</param>
        private void ImageSample_Activated(object sender, EventArgs e)
        {
            if (bPortDialogCalled)
                return;
            bPortDialogCalled = true;
            // If this is true, then we are running in VS2005 designer.
            if (AppDomain.CurrentDomain.FriendlyName != "DefaultDomain")
            {
                // Set the default commPort
                SampleImageControl.commPort = commPorts.DefaultCommPort; ;
                // Load the current port and intialize the filter list into the select comm port dialog.
                selectPortDlg.ActiveFilterList = (SelectCommPortDialog.PortFilters.SerialPorts | SelectCommPortDialog.PortFilters.USBPorts);
                selectPortDlg.commPort = SampleImageControl.commPort;
                // Make sure the screen is fully drawn.
                Refresh();
                // Show the dialog. It will return OK if the apply button is selected.
                if (selectPortDlg.ShowDialog() == DialogResult.OK)
                {
                    // If this is a new port, prepare to disconnect from current then reconnect.
                    // Set the new port
                    SampleImageControl.commPort = selectPortDlg.commPort;
//                    IsSerialPort(SampleImageControl.commPort);
                    StatusBox.Text = string.Format("Connecting To Imager On {0}", SampleImageControl.commPort.ToString());
                    ConnectTimer.Enabled = true;
                }
                // We're done with the commPorts class so we can release the memory.
                commPorts.Dispose();
            }
        }        /// <summary>
        /// If exit selected, call Form close.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnExit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Activate the SelectCommPort dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSelectCommPort(object sender, EventArgs e)
        {
            // Show the dialog. It will return OK if the apply button is selected.
            if (selectPortDlg.ShowDialog() == DialogResult.OK)
            {
                // If this is a new port, prepare to disconnect from current then reconnect.
                if (SampleImageControl.commPort != selectPortDlg.commPort)
                {
                    // Set the new port
                    SampleImageControl.commPort = selectPortDlg.commPort;
                    // Check to see if port is a serial port
                    bIsSerialPort = (string.Compare(selectPortDlg.currentPortType, "Serial") == 0) ? true : false;
                    // Recycle the connection to disconnect from the current port and connect to the new port.
                    OnRecycleConnection(this, new EventArgs());
                }
            }
            SampleImageControl.Focus();
        }
        /// <summary>
        /// Recyle the current connection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnRecycleConnection(object sender, EventArgs e)
        {
            // If currently connect, disconnect.
            if (SampleImageControl.IsConnected())
            {
                // Display disconnecting message.
                StatusBox.Text = "Disconnecting";
                StatusBox.Refresh();
                // disconnect from the imager.
                try
                {
                    SampleImageControl.Disconnect();
                }
                catch (ImagerException ex)
                {
                    // Pretty much ignore any exceptions in case the problem is a disconnected imager.
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
            }
            // Start the reconnect timer.
            StatusBox.Text = "Reconnecting Imager...";
            ConnectTimer.Enabled = true;
        }
        /// <summary>
        /// Display the about box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnAbout(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        /// <summary>
        /// Image capture button selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCaptureImage(object sender, EventArgs e)
        {
            if (!bImageCapture)
            {
                try
                {
                    // Set status message.
                    StatusBox.Text = "Capturing Image...";
                    Refresh();
                    // Call the control's non-blocking image capture method.
                    SampleImageControl.CaptureImageNoWait(null, null);
                    bImageCapture = true;
                }
                catch (ImagerException ex)
                {
                    SampleImageControl.PlaySounds(false);
                    StatusBox.Text = "Capture Image Failed: " + ex.Message;
                }
                finally
                {
                    SampleImageControl.Focus();
                }
            }
        }
        /// <summary>
        /// Clears the picture box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnClearView(object sender, EventArgs e)
        {
            SampleImageControl.Image = null;
            SampleImageControl.Focus();
        }
        /// <summary>
        /// A timer is used to start connect so that the gui has time to redraw on app start or recycle.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        private void OnConnectImager(object sender, EventArgs e)
        {
            ConnectTimer.Enabled = false;
            try
            {
                ConnectToImager();
            }
            catch (ImagerException ex)
            {
                StatusBox.Text = ("Connect Failed" + ex.Message);
            }
        }
        /// <summary>
        /// The autocrop checkbox check state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBoxAutoCrop_Click(object sender, EventArgs e)
        {
            SampleImageControl.imgTrans.AutoCrop = checkBoxAutoCrop.Checked;
        }
        /// <summary>
        /// The user has selected a new image display mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBoxDisplayMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxDisplayMode.SelectedIndex == 0)
                SampleImageControl.ImageDisplay = ImageControl.ImageDisplayMode.Normal;
            else
                SampleImageControl.ImageDisplay = ImageControl.ImageDisplayMode.ScaledToFit;
        }
        /// <summary>
        /// Display the image transfer progress in the status bar.
        /// </summary>
        /// <param name="theImageControl"></param>
        /// <param name="transferProgress"></param>
        private void SampleImageControl_TransferProgressEvent(object theImageControl, CTransferProgress transferProgress)
        {
            StatusBox.Text = string.Format("Image Transfer {0}% Complete", transferProgress.PercentComplete.ToString());
            Refresh();
        }
        /// <summary>
        /// Called when a data event is received by the control.  We look for the image event and the image capture result.
        /// </summary>
        /// <param name="theImageControl"></param>
        /// <param name="dataEvent">DataEvent instance for the event.</param>
        private void SampleImageControl_DataEvent(object theImageControl, DataEvent dataEvent)
        {
            if (dataEvent.EventType == EventType.ET_IMAGE)
            {
                string strTmp = "Image Capture Successful.";
                bool bSuccess = true;
                if (dataEvent.nResult != ImagerResult.RESULT_SUCCESS)
                {
                    bSuccess = false;
                    strTmp = "Image Capture Failed: " + SampleImageControl.imgrCaps.GetErrorString(dataEvent.nResult);
                }
                SampleImageControl.PlaySounds(bSuccess);
                StatusBox.Text = strTmp;
                bImageCapture = false;

            }
        }

        #endregion
    }
}
