/*
******************************************************************************
*
*     Unless  otherwise agreed to by Hand Held Products and the user of this 
* code, this code and information is provided "as is" without warranty or any
* kind, either expressed or implied, including but not limited to the implied
* warranties of merchantability and/or fitness for a particular purpose.
*
*             Copyright (c) 2005-2006 Hand Held Products, Inc.
*
*                         All Rights Reserved.
*
******************************************************************************
*
* $RCSfile: Library_Tools/CRegistry.cpp $
* $Revision: 1.1 $
* $Date: 2007/04/24 08:33:15EDT $
* $Author: MontoroJ $
*
*  File:        CRegistry.cpp
*
*  Description: This file contains the methods for the class CRegistry
*               which allows the user to read/set NT registry values.
*
*  Notes:       These methods use functions specific to Windows > 3.1.
*
*  Functions:   Public Interface
*               ----------------
*               CRegistry::CRegistry()      Overloaded 1x
*               CRegistry::~CRegistry()
*               CRegistry::GetLastError()   Overloaded by inline.
*               CRegistry::SetRegistryPath()
*               CRegistry::GetValue()       Overloaded 1x
*               CRegistry::SetValue()
*               CRegistry::GetSubKeys()
*               CRegistry::GetAllValues()
*               CRegistry::DeleteValue()
*               CRegistry::DeleteKey()
*
*               Private Interface
*               -----------------
*               CRegistry::OpenRegistryPath()
*               CRegistry::DeleteCurrentPath()
*
*  Rev. Date    By     Revision Notes
*  ----------   ---    ----------------
*  10/01/2002   JCM    Original Release (Modified Imager 8000 code)
*
******************************************************************************
*/

#include    <windows.h>

#ifndef _WIN32_WCE
    #include <TChar.h>
    #if DEBUG
        #include <stdio.h>
    #endif
#endif
//  #include    <stdlib.h>
//  #include    <string.h>
//  #include    <winioctl.h>

#include    "CRegistry.h"

// Levels to our variables
const DWORD PATH_LEVELS = 2;
static HKEY   g_defaultKey = HKEY_LOCAL_MACHINE;
// The default registry path to our archive stuff
static PTCHAR g_pRegistryLevels[ PATH_LEVELS ] =
{
    _T("Software"),
    _T("HHP, Inc.")
};

/*
******************************************************************************
*
*  Name:        CRegistry::CRegistry()
*
*  Description: Sets path to default registry key or to the specified key but
*               does not open it.
*
*  Parameters:  None.
*
*  Returns:     None.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

CRegistry::CRegistry( VOID )
{
    DWORD   dwLen = 0;  // String Length

    // Initialize member variables.
    m_bRegistryOpen = FALSE;
    m_bReadOnly     = FALSE;
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Initialize the key & path members to our defaults
    m_hkMainRegKey = g_defaultKey;
    for( m_nPathLevels = 0; m_nPathLevels < PATH_LEVELS; m_nPathLevels++ )
    {
        dwLen = (DWORD)::_tcsclen( g_pRegistryLevels[m_nPathLevels] ) + 1;
        m_chPaths[m_nPathLevels] = new TCHAR[ dwLen ];
        if( m_chPaths[ m_nPathLevels ] != NULL )
            ::_tcscpy( m_chPaths[m_nPathLevels],g_pRegistryLevels[m_nPathLevels] );
        else
        {
            m_dwSysError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
            break;
        }
    }
    // If allocation was not successful.
    if( m_nPathLevels != PATH_LEVELS )
    {
        while( m_nPathLevels-- > 0 )
            delete [] m_chPaths[m_nPathLevels];
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::CRegistry() - Error Setting Default Path: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }
}
// End CRegistry()

/*
******************************************************************************
*
*  Name:        CRegistry::CRegistry()
*
*  Description: Sets path to supplied registry key or to the specified key but
*               does not open it.
*
*  Parameters:  None.
*
*  Returns:     None.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

CRegistry::CRegistry( HKEY key,PTCHAR *pPath,int nLevels )
{
    DWORD   dwLen = 0;  // String Length

    // Initialize member variables.
    m_bRegistryOpen = FALSE;
    m_bReadOnly     = FALSE;
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Validate paramters.
    if( (pPath == NULL) || (nLevels > MAX_PATH_DEPTH) )
    {
        m_dwRegError = REG_ERROR_CONSTRUCTOR_BAD_PARAMETER;
        return;
    }
    // Initialize the key & path members to our defaults
    m_hkMainRegKey = key;
    for( m_nPathLevels = 0; m_nPathLevels < nLevels; m_nPathLevels++ )
    {
        dwLen = (DWORD)::_tcsclen( pPath[m_nPathLevels] ) + 1;
        m_chPaths[m_nPathLevels] = new TCHAR[ dwLen ];
        if( m_chPaths[ m_nPathLevels ] != NULL )
            ::_tcscpy( m_chPaths[m_nPathLevels],pPath[m_nPathLevels] );
        else
        {
            m_dwSysError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
            break;
        }
    }

    // If allocation was not successful.
    if( m_nPathLevels != nLevels )
    {
        while( m_nPathLevels-- > 0 )
            delete [] m_chPaths[m_nPathLevels];
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::CRegistry() - Error Setting User Path: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }
}
// End CRegistry()

/*
******************************************************************************
*
*  Name:        CRegistry::~CRegistry()
*
*  Description: The destructor closes the path if its open.
*
*  Parameters:  None.
*
*  Returns:     None.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

CRegistry::~CRegistry()
{
    // If the registry is open, close the handle
    if( m_bRegistryOpen )
        ::RegCloseKey( m_hkRegistry );
    // Delete the current path
    DeleteCurrentPath();
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
}
// End ~CRegistry()

/*
******************************************************************************
*
*  Name:        CRegistry::GetLastError()
*
*  Description: Returns both the PSC error and the system error.
*
*  Parameters:  PDWORD  pdwSysErr; (O) // System error (::GetLastError())
*
*  Returns:     Value of m_dwRegError;
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

DWORD CRegistry::GetLastError( PDWORD pdwSysError )
{
    // Return the system error.
    if( pdwSysError != NULL )
        *pdwSysError = m_dwSysError;

    return( m_dwRegError );
}
// End GetLastError()

/*
******************************************************************************
*
*  Name:        CRegistry::SetRegistryPath()
*
*  Description: Set the specified path to the member m_chPaths & open it.
*
*  Parameters:  HKEY  key;           (I)   Main registry folder
*               const PTCHAR *pPath; (I)   Path to values in folder
*               int   nLevels;       (I)   Levels in path
*
*  Returns:     TRUE  - Registry open with new path.
*               FALSE - Registry closed, path not set. Check get error
*                       function for reason.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::SetRegistryPath( HKEY key,PTCHAR *pPath,int nLevels )
{
    DWORD   dwLen = 0;  // String Length

    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Validate paramters.
    if( (pPath == NULL) || (nLevels > MAX_PATH_DEPTH) )
    {
        m_dwRegError = REG_ERROR_SET_PATH_BAD_PARAMETER;
        return( FALSE );
    }
    // Delete the current path.
    DeleteCurrentPath();
    // Set the new path to the member path list.
    m_hkMainRegKey = key;
    for( m_nPathLevels = 0; m_nPathLevels < nLevels; m_nPathLevels++ )
    {
        dwLen = (DWORD)::_tcsclen( pPath[m_nPathLevels] ) + 1;
        m_chPaths[m_nPathLevels] = new TCHAR[ dwLen ];
        if( m_chPaths[ m_nPathLevels ] != NULL )
            ::_tcscpy( m_chPaths[m_nPathLevels],pPath[m_nPathLevels] );
        else
        {
            m_dwSysError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
            break;
        }
    }
    // If allocation was not successful.
    if( m_nPathLevels != nLevels )
    {
        while( m_nPathLevels-- > 0 )
            delete [] m_chPaths[m_nPathLevels];
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::SetRegistryPath() - Error Setting Path: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }

    return( OpenRegistryPath() );
}
// End SetRegistryPath()

/*
******************************************************************************
*
*  Name:        CRegistry::SetRegistryPath()
*
*  Description: Set the specified path to the member m_chPaths & open it.
*
*  Parameters:  HKEY  key;           (I)   Main registry folder
*               const PTCHAR *pPath; (I)   Path to values in folder
*               int   nLevels;       (I)   Levels in path
*
*  Returns:     TRUE  - Registry open with new path.
*               FALSE - Registry closed, path not set. Check get error
*                       function for reason.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::SetRegistryPath( HKEY key,PTCHAR ptcPath )
{
    DWORD   dwLen = 0;  // String Length

    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Validate paramters.
    if( ptcPath == NULL )
    {
        m_dwRegError = REG_ERROR_SET_PATH_BAD_PARAMETER;
        return( FALSE );
    }
    // Delete the current path.
    DeleteCurrentPath();
    // Set the new path to the member path list.
    m_hkMainRegKey = key;
    m_nPathLevels = ParsePath( m_chPaths,ptcPath );

    return( OpenRegistryPath() );
}
// End SetRegistryPath()

/*
******************************************************************************
*
*  Name:        CRegistry::RegistryPathExists()
*
*  Description: Verify that the specified registry path already exists.  
*
*  Parameters:  HKEY  key;           (I)   Main registry folder
*               const PTCHAR *pPath; (I)   Path to values in folder
*               int   nLevels;       (I)   Levels in path
*
*  Returns:     TRUE  - Full path exists in registry.
*               FALSE - Full path does not exists. Check get error function for 
*                       reason.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  09/16/2004   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::RegistryPathExists( HKEY hKey,PTCHAR *ptcPaths,int nNumPaths )
{
    BOOL    bRetVal = TRUE; // Assume Path exists.
    HKEY    key1;           // Handles returned & sent to Open Key call
    HKEY    key2;

    // If the registry is open, close the handle
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Open the first path with the base key
    if( ERROR_SUCCESS != (m_dwSysError = ::RegOpenKeyEx( hKey,ptcPaths[0],0,KEY_READ,&key1 )) )
    {
        m_dwRegError = REG_ERROR_SET_PATH_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::RegistryPathExists() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        bRetVal = FALSE;
    }
    // Open the subsequent paths using the previous key
    for( int i = 1; (i < nNumPaths) && bRetVal; i++ )
    {
        key2 = key1;
        if( ERROR_SUCCESS != (m_dwRegError = ::RegOpenKeyEx( key2,ptcPaths[i],0,KEY_READ,&key1 ) ) )
        {
            ::RegCloseKey( key2 );
            m_dwRegError = REG_ERROR_SET_PATH_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::RegistryPathExists() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
            bRetVal = FALSE;
        }
        else
            ::RegCloseKey( key2 );
    }
    // If opened okay we have to close key1
    if( bRetVal )
        ::RegCloseKey( key1 );

    return( bRetVal );
}
// End RegistryPathExists()

/*
******************************************************************************
*
*  Name:        CRegistry::RegistryPathExists()
*
*  Description: Verify that the specified registry path already exists.  
*
*  Parameters:  HKEY  key;           (I)   Main registry folder
*               const PTCHAR *pPath; (I)   Path to values in folder
*               int   nLevels;       (I)   Levels in path
*
*  Returns:     TRUE  - Full path exists in registry.
*               FALSE - Full path does not exists. Check get error function for 
*                       reason.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  09/16/2004   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::RegistryPathExists( HKEY hKey,PTCHAR ptcPath )
{
    BOOL    bRetVal = TRUE; // Assume Path exists.
    HKEY    key1;           // Handles returned & sent to Open Key call
    HKEY    key2;
    PTCHAR  ptcPaths[ MAX_PATH_DEPTH ];

    // If the registry is open, close the handle
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;


    int nNumPaths = ParsePath( ptcPaths,ptcPath );
    // Open the first path with the base key
    if( ERROR_SUCCESS != (m_dwSysError = ::RegOpenKeyEx( hKey,ptcPaths[0],0,KEY_READ,&key1 )) )
    {
        m_dwRegError = REG_ERROR_SET_PATH_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::RegistryPathExists() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        bRetVal = FALSE;
    }
    // Open the subsequent paths using the previous key
    for( int i = 1; (i < nNumPaths) && bRetVal; i++ )
    {
        key2 = key1;
        if( ERROR_SUCCESS != (m_dwRegError = ::RegOpenKeyEx( key2,ptcPaths[i],0,KEY_READ,&key1 ) ) )
        {
            ::RegCloseKey( key2 );
            m_dwRegError = REG_ERROR_SET_PATH_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::RegistryPathExists() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
            bRetVal = FALSE;
        }
        else
            ::RegCloseKey( key2 );
    }
    // If opened okay we have to close key1
    if( bRetVal )
        ::RegCloseKey( key1 );

    for( int n = 0; n < nNumPaths; n++ )
        delete [] ptcPaths[n];

    return( bRetVal );
}
// End RegistryPathExists()

/*
******************************************************************************
*
*  Name:        CRegistry::GetValue()
*
*  Description: Calls overloaded GetValue() supplying length and type vars.
*
*  Parameters:  const  PTCHAR name; (I)  Variable name in registry
*               PVOID  value;       (O)  Void * where data returned
*               int    maxLen;      (I)  Maximum allowef size of data
*
*  Returns:     TRUE  - If data read okay.
*               FALSE - If registry not open or data not found.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::GetValue( const PTCHAR pName,PVOID pValue,int nMaxLen )
{
    DWORD   dwLen;
    DWORD   dwType;

    return( GetValue( pName,pValue,&dwLen,&dwType,nMaxLen ) );
}
// End GetValue()

/*
******************************************************************************
*
*  Name:        CRegistry::GetValue()
*
*  Description: Attempts to read the data in "name".  If successful the data
*               is returned in value and the function returns TRUE. Otherwise,
*               the return value is FALSE.
*
*  Parameters:  const  PTCHAR name; (I)  Variable name in registry
*               PVOID  value;       (O)  Void * where data returned
*               PDWORD dwLen;       (O)  Size of data in bytes.
*               PDWORD dwType;      (O)  Registry Data Type
*               int    maxLen;      (I)  Maximum allowef size of data
*
*  Returns:     TRUE  - If data read okay.
*               FALSE - If registry not open or data not found.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::GetValue( const  PTCHAR pName,
                          PVOID  pValue,
                          PDWORD pdwLen,
                          PDWORD pdwType,
                          int    nMaxLen )
{

    PBYTE   pBuff;              // Local for value
    BOOL    bRetVal = FALSE;    // TRUE if okay, FALSE if failed

    // If the registry is not open, attempt to open it
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if( !m_bRegistryOpen )
    {
        if( !OpenRegistryPath() )
            return( FALSE );
    }
    // Verify parameters.
    if( (pName == NULL) || (pValue == NULL) || (pdwLen == NULL) || (pdwType == NULL) )
    {
        m_dwRegError = REG_ERROR_GET_VALUE_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( FALSE );
    }
    // Attempt to read the value requested
    *pdwLen = (DWORD)nMaxLen;
    pBuff = (BYTE *)pValue;
    m_dwSysError = ::RegQueryValueEx( m_hkRegistry,pName,NULL,pdwType,pBuff,pdwLen );
    // If we read name okay, return TRUE
    if( m_dwSysError == ERROR_SUCCESS )
        bRetVal = TRUE;
    else
    {
        m_dwRegError = REG_ERROR_QUERY_VALUE_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }

    return( bRetVal );
}
// End GetValue()

/*
******************************************************************************
*
*  Name:        CRegistry::SetValue()
*
*  Description: Attempts to set the data in "name" to the registry. If
*               successfull the function returns TRUE.  Otherwise, the return
*               value is FALSE.
*
*  Parameters:  const PTCHAR name; (I)   Name of variable in registry
*               PVOID value;       (O)   Void type of data so set
*               int   len;         (I)   Size of data
*               DWORD dtype        (I)   Type of data
*
*  Returns:     TRUE  - If data value set okay.
*               FALSE - If registry not open or data not set.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::SetValue( const PTCHAR pName,PVOID pValue,int nLen,DWORD dwType )
{
    BYTE    *pBuff;             // Local for value
    BOOL    bRetVal = FALSE;    // TRUE if okay, FALSE if failed

    // If the registry is not open, attempt to open it
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if( !m_bRegistryOpen )
    {
        if ( !OpenRegistryPath() )
            return( FALSE );
    }
    // Verify pointer parameters
    if( (pName == NULL) || (pValue == NULL) )
    {
        m_dwRegError = REG_ERROR_SET_VALUE_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::SetValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( FALSE );
    }
    // Attempt to set the value
    pBuff = (BYTE *)pValue;
    m_dwSysError = ::RegSetValueEx( m_hkRegistry,
                                    pName,
                                    NULL,
                                    dwType,
                                    pBuff,
                                    (DWORD)nLen );
    // If we read name okay, return TRUE
    // If we read name okay, return TRUE
    if( m_dwSysError == ERROR_SUCCESS )
        bRetVal = TRUE;
    else
    {
        m_dwRegError = REG_ERROR_SET_VALUE_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::SetValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }

    return( bRetVal );
}
// End SetValue()

/*
******************************************************************************
*
*  Name:        CRegistry::GetSubKeys()
*
*  Description: Retrieves all the subkey names and class if requested of all
*               the registry keys one level under the current key. Memory is '
*               allocated for each name as needed.
*
*               NOTE: It is the callers responsibility to delete the strKeys
*               and strClass buffers.
*
*  Parameters:  PTCHAR *strKeys;  (O)   Names of subkeys
*               PTCHAR *strClass; (O)   Class of keys (not returned if NULL)
*               int   nMaxKeys;   (I)   Max number of keys.
*
*  Returns:     Number of Keys return ed if successful.
*               -1 if error. Check reg.getLastError().
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

int CRegistry::GetSubKeys( PTCHAR *strKeys,PTCHAR *strClass,int nMaxKeys )
{
    BOOL     bStoreClass    = FALSE;            // Store class as well
    int      nNumKeys       = 0;                // Number of keys
    DWORD    dwEnumRetVal   = 0;                // Return value.
    DWORD    dwEnumNameLen  = MAX_KEY_NAME_LEN; // Enumeration name len
    DWORD    dwClassNameLen = 0;                // Class name len
    PDWORD   pdwClassNameLen = NULL;            // Pointer to Class name len
    PTCHAR   strKeyTmp      = NULL;             // Key name pointer.
    PTCHAR   strClassTmp    = NULL;             // Class name pointer.
#ifndef _WIN32_WCE
    FILETIME lastWrite;                         // Last update time.
#endif

    // Make sure keys string array is valid.
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if( strKeys == (PTCHAR *)NULL )
    {
        m_dwRegError = REG_ERROR_GET_SUBKEYS_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetSubKeys() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( -1 );
    }
    // Allocate a local buffer for class name if classes requested
    // i.e. strClass is not NULL.
    if( strClass != (PTCHAR *)NULL )
    {
        if( (strClassTmp = new TCHAR[ MAX_CLASS_NAME_LEN ]) == NULL )
        {
            m_dwSysError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
        }
        else
        {
            bStoreClass = TRUE;
            dwClassNameLen = MAX_CLASS_NAME_LEN;
            pdwClassNameLen = &dwClassNameLen;
        }
    }
    if( m_dwRegError == REG_ERROR_SUCCESS )
    {
        // Allocate local buffer for key name, WOOF if allocation fails.
        if( (strKeyTmp = new TCHAR[ MAX_KEY_NAME_LEN ]) == NULL )
        {
            m_dwSysError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
        }
        else
        {
            // Get first sub key
            dwEnumRetVal = ::RegEnumKeyEx( m_hkRegistry,
                                           nNumKeys,
                                           strKeyTmp,
                                           &dwEnumNameLen,
                                           NULL,
                                           strClassTmp,
                                           pdwClassNameLen,
#ifdef _WIN32_WCE
                                           NULL );
#else
                                           &lastWrite );
#endif
            // Loop while get next sub key succeeds
            while( (dwEnumRetVal == ERROR_SUCCESS) && (nNumKeys < nMaxKeys) )
            {
                // allocate enough space in array for string plus NULL. If allocation
                // fails, WOOF! Otherwise, store key.
                if( (strKeys[ nNumKeys ] = new TCHAR[ (dwEnumNameLen + 1) ]) == NULL )
                {
                    m_dwSysError = ::GetLastError();
                    m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
                    break;
                }
                ::_tcscpy( strKeys[ nNumKeys ],strKeyTmp );
                // Do same as above to class is strClass is not NULL
                if( bStoreClass )
                {
                    if( dwClassNameLen > 0 )
                    {
                        if( (strClass[ nNumKeys ] = new TCHAR[ (dwClassNameLen + 1) ]) == NULL )
                        {
                            delete [] strKeys[ nNumKeys ];
                            m_dwSysError = ::GetLastError();
                            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
                            break;
                        }
                        ::_tcscpy( strClass[ nNumKeys ],strClassTmp );
                        dwClassNameLen  = MAX_CLASS_NAME_LEN;
                    }
                }
                // Get next key.
                dwEnumNameLen = MAX_KEY_NAME_LEN;
                dwEnumRetVal = ::RegEnumKeyEx( m_hkRegistry,
                                               ++nNumKeys,
                                               strKeyTmp,
                                               &dwEnumNameLen,
                                               NULL,
                                               strClassTmp,
                                               pdwClassNameLen,
#ifdef _WIN32_WCE
                                               NULL );
#else
                                               &lastWrite );
#endif
            }   // End while loop
        }
    }
    // Error clean up
    if( m_dwRegError != REG_ERROR_SUCCESS )
    {
        // Delete any key & class strings that were allocated.
        if( nNumKeys > 0 )
        {
            for( int i = 0; i < nNumKeys; i++ )
            {
                delete [] strKeys[i];
                if( bStoreClass )
                    delete [] strClass[i];
            }
        }
        // Set return value to -1
        nNumKeys = -1;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetSubKeys() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }
    // Clean up Key temp storage
    if( strKeyTmp != NULL )
        delete [] strKeyTmp;
    // Clean up class tmp storage
    if( strClassTmp != NULL )
        delete [] strClassTmp;

    return( nNumKeys );
}
// End GetSubKeys()

/*
******************************************************************************
*
*  Name:        CRegistry::GetAllValues())
*
*  Description: Retrieves all the value names, data type and data for all the
*               registry values one level under the current key. Memory is
*               allocated for each name as needed.
*
*               **NOTE** It is the callers responsibility to delete the
*               strValueNames and valueData buffer string arrays.
*
*  Parameters:  PTCHAR *strValueNames; (O) Names of values
*               PDWORD dwDataTypes;    (O) Data types of values.
*               PBYTE  *valueData;     (O) Value data buffers
*               PDWORD dwDataLen;      (O) Length of data buffers
*               int    nMaxvalues;     (I) Max number of values.
*
*  Returns:     Number of values returned if successful.
*               -1 if error. Check reg.getLastError().
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

int CRegistry::GetAllValues( PTCHAR *strValueNames,
                             PDWORD pdwDataTypes,
                             PBYTE  *valueData,
                             PDWORD pdwDataLen,
                             int    nMaxValues )
{
    DWORD   dwGetEnumRet = 0;    // Return value.
    DWORD   dwType       = 0;    // Value type.
    DWORD   dwValueNameLen = 0;  // Length of value name.
    DWORD   dwValueDataLen = 0;  // Length of data.
    PTCHAR  strValue     = NULL; // Value name string.
    PBYTE   data         = NULL; // Data.
    int     numValues    = 0;    // Number of values.
    BOOL    bCopyValues  = TRUE; // Save value types.

    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    // Make sure string name array is valid pointer
    if( strValueNames == NULL )
    {
        m_dwRegError = REG_ERROR_GET_ALL_VALUES_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetAllValues() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( -1 );
    }
    // If any of data pointers are NULL, they all should be NULL. If not
    // there is an error.
    if( (valueData == NULL) || (pdwDataLen == NULL) )
    {
        if( (valueData == NULL) && (pdwDataLen == NULL) )
        {
            bCopyValues = FALSE;
        }
        else
        {
            m_dwRegError = REG_ERROR_GET_ALL_VALUES_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetAllValues() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
            return( -1 );
        }
    }
    // Allocate local buffers for Value name and Value data.  WOOF if
    // memory allocation fails.
    if( (strValue = new TCHAR[ MAX_VALUE_NAME_LEN ]) == NULL )
    {
        m_dwSysError = ::GetLastError();
        m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
    }
    else
    {
        dwValueNameLen = MAX_VALUE_NAME_LEN;
        if( (data = new BYTE[ MAX_VALUE_DATA_LEN ]) == NULL )
        {
            m_dwRegError = ::GetLastError();
            m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
        }
        else
        {
            dwValueDataLen = MAX_VALUE_DATA_LEN;
            // Get First value
            dwGetEnumRet = ::RegEnumValue( m_hkRegistry,
                                           numValues,
                                           strValue,
                                           &dwValueNameLen,
                                           NULL,
                                           &dwType,
                                           data,
                                           &dwValueDataLen );
            // Loop until no more values or error or max num values read.
            while( (dwGetEnumRet == ERROR_SUCCESS) && (numValues < nMaxValues) )
            {
                // Store value name. If buffer allocation fails, WOOF!
                strValueNames[ numValues ] = new TCHAR[ (dwValueNameLen + 1) ];
                if( strValueNames[ numValues ] == NULL )
                {
                    m_dwRegError = ::GetLastError();
                    m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
                    break;
                }
                ::_tcscpy( strValueNames[ numValues ],strValue );
                // If value type array is not NULL, copy in value type
                if( pdwDataTypes != NULL )
                    pdwDataTypes[ numValues ] = dwType;
                // If copy values
                if( bCopyValues )
                {
                    // Store value data. If buffer allocation fails, WOOF!
                    if( (valueData[ numValues ] = new BYTE[ dwValueDataLen ]) == NULL )
                    {
                        delete [] strValueNames[ numValues ];
                        m_dwSysError = ::GetLastError();
                        m_dwRegError = REG_ERROR_MEMORY_ALLOCATION_FAILED;
                        break;
                    }
                    ::memcpy( valueData[ numValues ],data,dwValueDataLen );
                    // Store data type and data length.
                    pdwDataLen[ numValues ] = dwValueDataLen;
                }
                // Get next value.
                dwValueNameLen = MAX_VALUE_NAME_LEN;
                dwValueDataLen = MAX_VALUE_DATA_LEN;
                dwGetEnumRet = ::RegEnumValue( m_hkRegistry,
                                               ++numValues,
                                               strValue,
                                               &dwValueNameLen,
                                               NULL,
                                               &dwType,
                                               data,
                                               &dwValueDataLen );
            }
        }
    }

    // Error recovery
    if( m_dwRegError != REG_ERROR_SUCCESS )
    {
        // Delete and allocated name or data buffers.
        if( numValues > 0 )
        {
            for( int i = 0; i < numValues; i++ )
            {
                delete [] strValueNames[i];
                if( bCopyValues )
                    delete [] valueData[i];
            }
        }
        // Set return value to -1
        numValues = -1;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::GetAllValues() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }
    // Delete temp Value string.
    if( strValue != NULL )
        delete [] strValue;
    // Delete temp Data string.
    if( data != (BYTE *)NULL )
        delete [] data;

    return( numValues );
}
// End GetAllValues()

/*
******************************************************************************
*
*  Name:        CRegistry::DeleteValue()
*
*  Description: Deletes a value from the current key.
*
*  Parameters:  const PTCHAR strValue; (I) Variable Name in registry.
*
*  Returns:     TRUE  - if value deleted successfully.
*               FALSE - if error. Check GetLastError().
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::DeleteValue( const PTCHAR strValue )
{
    BOOL    bRetVal = FALSE;    // Return value

    // If the registry is not open, attempt to open it
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if( !m_bRegistryOpen )
    {
        if( !OpenRegistryPath() )
            return( FALSE );
    }
    // Make sure name is not NULL string
    if( strValue == (PTCHAR)NULL )
    {
        m_dwRegError = REG_ERROR_DELETE_VALUE_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::DeleteValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( FALSE );
    }
    // Delete the value
    m_dwSysError = ::RegDeleteValue( m_hkRegistry,strValue );
    // Return TRUE only if no error.
    if( m_dwSysError == ERROR_SUCCESS )
        bRetVal = TRUE;
    else
    {
        m_dwRegError = REG_ERROR_DELETE_VALUE_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::DeleteValue() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }

    return( bRetVal );
}
// End DeleteValue()

/*
******************************************************************************
*
*  Name:        CRegistry::DeleteKey()
*
*  Description: Deletes a subkey from the current key. The key to delete must
*               be empty first.
*
*  Parameters:  const PTCHAR strKey; (I) Key Name in registry.
*
*  Returns:     TRUE  - if value deleted successfully.
*               FALSE - if error. Check GetLastError().
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::DeleteKey( const PTCHAR strKey )
{
    BOOL    bRetVal = FALSE;    // Return value

    // If the registry is not open, attempt to open it
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if ( !m_bRegistryOpen )
    {
        if ( !OpenRegistryPath() )
            return( FALSE );
    }
    // Make sure name is not NULL string
    if( strKey == (const PTCHAR)NULL )
    {
        m_dwRegError = REG_ERROR_DELETE_KEY_BAD_PARAMETER;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::DeleteKey() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( FALSE );
    }
    // Delete the value
    m_dwSysError = ::RegDeleteKey( m_hkRegistry,strKey );
    // Return TRUE only if no error.
    if( m_dwSysError == ERROR_SUCCESS )
        bRetVal = TRUE;
    else
    {
        m_dwRegError = REG_ERROR_DELETE_KEY_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::DeleteKey() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
    }

    return( bRetVal );
}
// End DeleteKey()

/*
******************************************************************************
*
*                           Private Methods
*
******************************************************************************
*/

/*
******************************************************************************
*
*  Name:        CRegistry::OpenRegistryPath()
*
*  Description: Open the registry path stored in the private member variables
*               m_hkMainRegKey, m_chPaths.
*
*  Parameters:  None - Uses member variables. setRegistryPath() Should be
*               called to change the path from the default.
*
*  Returns:     TRUE  - If path opened okay.
*               FALSE - If path not open.  User should call member
*                       GetLastError() to find out why. (winerror.h)
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/

BOOL CRegistry::OpenRegistryPath( VOID )
{
    HKEY    key1;        // Handles returned & sent to Create Key call
    HKEY    key2;
    DWORD   dwDisp;      // Address of dwDisposition value buffer
    DWORD   dwKeyFlags;  // Key access flags

    // If the registry is open, close the handle
    m_dwRegError = REG_ERROR_SUCCESS;
    m_dwSysError = ERROR_SUCCESS;
    if( m_bRegistryOpen )
    {
        ::RegCloseKey( m_hkRegistry );
        m_bRegistryOpen = FALSE;
    }
    // Open the first path with the base key
    if( ERROR_SUCCESS != (m_dwSysError = ::RegCreateKeyEx( m_hkMainRegKey,
                                                           m_chPaths[0],
                                                           0L,
                                                           NULL,
                                                           REG_OPTION_NON_VOLATILE,
                                                           KEY_READ,
                                                           NULL,
                                                           &key1,
                                                           &dwDisp )) )
    {
        m_dwRegError = REG_ERROR_CREATE_KEY_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::OpenRegistryPath() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
        return( FALSE );
    }
    // Open the subsequent paths using the previous key
    dwKeyFlags = ( m_bReadOnly ) ? KEY_READ : (KEY_READ | KEY_WRITE);
    for( int i = 1; i < m_nPathLevels; i++ )
    {
        key2 = key1;
        if( ERROR_SUCCESS != (m_dwRegError = ::RegCreateKeyEx( key2,
                                                               m_chPaths[i],
                                                               0L,
                                                               NULL,
                                                               REG_OPTION_NON_VOLATILE,
                                                               dwKeyFlags,
                                                               NULL,
                                                               &key1,
                                                               &dwDisp ) ) )
        {
            ::RegCloseKey( key2 );
            m_dwRegError = REG_ERROR_CREATE_KEY_FAILED;
#ifdef DEBUG
        ::_stprintf( m_tcDebugStr,_T("  CRegistry::OpenRegistryPath() - Error: %x - %d"),m_dwRegError,m_dwSysError );
        ::OutputDebugString( m_tcDebugStr );
#endif
            return( FALSE );
        }
        ::RegCloseKey( key2 );
    }
    // Set global key handle
    m_hkRegistry = key1;
    // Set count to TRUE == 1
    m_bRegistryOpen = TRUE;

    return( TRUE );
}
// End OpenRegistryPath()

/*
******************************************************************************
*
*  Name:        CRegistery::DeleteCurrentPath()
*
*  Description: Deletes the buffers allocated in m_chPaths.
*
*  Parameters:  None.
*
*  Returns:     None.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  02/09/2000   JCM   Original Release
*
******************************************************************************
*/
VOID CRegistry::DeleteCurrentPath( VOID )
{
    for( int i = 0; i < m_nPathLevels; i++ )
        delete [] m_chPaths[i];
    m_nPathLevels = 0;

    return;
}
// End DeleteCurrentPath()

int CRegistry::ParsePath( PTCHAR paths[ MAX_PATH_DEPTH ],PTCHAR ptcPath )
{
    int     nNumLevels = 0;
    TCHAR   tcTmp[ MAX_PATH ];
    int     nTmp = 0;

    if( ptcPath == NULL )
        return( -1 );

    int nLen = (int)::_tcslen( ptcPath );
    if( nLen == 0 )
        return( 0 );

    for( int n = 0; n < nLen; n++ )
    {
        if( ptcPath[n] == _T('\\') )
        {
            tcTmp[ nTmp++ ] = 0;
            paths[ nNumLevels ] = new TCHAR[ nTmp ];
            ::_tcscpy( paths[ nNumLevels++ ],tcTmp );
            nTmp = 0;
        }
        else
            tcTmp[ nTmp++ ] = ptcPath[n];
    }
    if( nTmp > 0 )
    {
        tcTmp[ nTmp++ ] = 0;
        paths[ nNumLevels ] = new TCHAR[ nTmp ];
        ::_tcscpy( paths[ nNumLevels++ ],tcTmp );
    }
    return( nNumLevels );
}

// $Log: Library_Tools/CRegistry.cpp  $
// Revision 1.1 2007/04/24 08:33:15EDT MontoroJ 
// Initial revision
// Member added to project d:/MKS_ES/ES/Matrix SDK/Matrix5080SDK/Matrix5080SDK.pj
// Revision 1.2 2004/09/17 15:20:33EDT montoroj 
// - Added variable names to prototypes.
// - Added method for determining whether a registry key exists.
// Revision 1.1 2003/06/19 11:54:34EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// Revision 1.1 2003/05/14 11:39:13EDT montoroj 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.1 2003/05/13 14:50:50EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.3 2003/02/20 16:33:34EST montoroj 
// - Replaced $LOG$ with $Log: Library_Tools/CRegistry.cpp  $
// - Replaced Revision 1.1 2007/04/24 08:33:15EDT MontoroJ 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_ES/ES/Matrix SDK/Matrix5080SDK/Matrix5080SDK.pj
// - Replaced Revision 1.2 2004/09/17 15:20:33EDT montoroj 
// - Replaced - Added variable names to prototypes.
// - Replaced - Added method for determining whether a registry key exists.
// - Replaced Revision 1.1 2003/06/19 11:54:34EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// - Replaced Revision 1.1 2003/05/14 11:39:13EDT montoroj 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// - Replaced Revision 1.1 2003/05/13 14:50:50EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj

