/*
******************************************************************************
*
*                   Copyright (c) 2002 HHP Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
* $RCSfile: Library_Tools/CRegistry.h $
* $Revision: 1.2 $
* $Date: 2008/01/15 09:37:09EST $
* $Author: MontoroJ $
*
*  File:        CRegistry.h
*
*  Description: Header file containing the declaration of class CRegistry.
*
*  Functions:   Public Interface
*               ----------------
*               CRegistry::SetReadOnly()
*               CRegistry::SetValueInt()
*               CRegistry::SetValueString()
*               CRegistry::GetLastError()
*
*               Private Interface
*               -----------------
*               None.
*
*  Rev. Date    By     Revision Notes
*  ----------   ---    ----------------
*  10/01/2002   JCM    Original Release (Modified from older Imager 8000 code)
*
******************************************************************************
*/

#ifndef _CREGISTRY_H

    #define _CREGISTRY_H

    #include    <stdio.h>
    #include    <TChar.h>   // For Unicode support.

    // Class interface to the registry. This is implemented as a class so that
    // the handle to the registry is closed properly with the destructor!

    #define MAX_PATH_DEPTH      16

    #define REG_INVALID_BUFFER  10001
    #define MAX_CLASS_NAME_LEN  64
    #define MAX_VALUE_NAME_LEN  64
    #define MAX_VALUE_DATA_LEN  2048
    #define MAX_KEY_NAME_LEN    64

    #define START_TYPE_BOOT     0
    #define START_TYPE_SYSTEM   1
    #define START_TYPE_AUTO     2
    #define START_TYPE_MANUAL   3
    #define START_TYPE_DISABLED 4

/*
*******************************************************************************
*                                                                             *
*                Data types for setValue() function.                          *
*                                                                             *
*                                                                             *
* REG_DWORD                 A 32-bit number.                                  *
* REG_DWORD_LITTLE_ENDIAN   A 32-bit number in little-endian format (same as  *
*                           REG_DWORD). In little-endian format, the most     *
*                           significant byte of a word is the high-order word.*
*                           This is the most common format for computers      *
*                           running Windows NT and Windows 95.                *
* REG_DWORD_BIG_ENDIAN      A 32-bit number in bigEndian format. In big-endian*
*                           format, the most significant byte of a word is the*
*                           low-order word.                                   *
* REG_SZ                    A null-terminated string. It will be a Unicode or *
*                           ANSI string depending on whether you use the Uni- *
*                           code or ANSI functions.                           *
* REG_EXPAND_SZ             A null-terminated string that contains unexpanded *
*                           references to environment variables (for example, *
*                           "%PATH%"). It will be a Unicode or ANSI string    *
*                           depending on whether you use the Unicode or ANSI  *
*                           functions.                                        *
* REG_BINARY                Binary data in any form.                          *
* REG_LINK                  A Unicode symbolic link.                          *
* REG_MULTI_SZ              An array of null-terminated strings, terminated by*
*                           two null characters.                              *
* REG_NONE                  No defined value type.                            *
* REG_RESOURCE_LIST         A device-driver resource list.                    *
*                                                                             *
*******************************************************************************
*/

/*
******************************************************************************
*
*                          Class CRegistry
*
******************************************************************************
*/
    class CRegistry
    {

    public:

        enum
        {
            REG_ERROR_SUCCESS=0,
            REG_ERROR_SET_PATH_BAD_PARAMETER,
            REG_ERROR_MEMORY_ALLOCATION_FAILED,
            REG_ERROR_CREATE_KEY_FAILED,
            REG_ERROR_QUERY_VALUE_FAILED,
            REG_ERROR_SET_VALUE_FAILED,
            REG_ERROR_DELETE_VALUE_FAILED,
            REG_ERROR_DELETE_KEY_BAD_PARAMETER,
            REG_ERROR_GET_VALUE_BAD_PARAMETER,
            REG_ERROR_SET_VALUE_BAD_PARAMETER,
            REG_ERROR_GET_SUBKEYS_BAD_PARAMETER,
            REG_ERROR_GET_ALL_VALUES_BAD_PARAMETER,
            REG_ERROR_DELETE_VALUE_BAD_PARAMETER,
            REG_ERROR_DELETE_KEY_FAILED,
            REG_ERROR_CONSTRUCTOR_BAD_PARAMETER,
            REG_ERROR_SET_PATH_FAILED
        };
        // member functions
        CRegistry();
        CRegistry( HKEY hKey,PTCHAR *ptcPaths,int nNumPaths );
       ~CRegistry();
        DWORD GetLastError( PDWORD pdwLastSysErr);
        BOOL  SetRegistryPath( HKEY hKey,PTCHAR ptcPath );
        BOOL  SetRegistryPath( HKEY hKey,PTCHAR *ptcPaths,int nNumPaths );
        BOOL  RegistryPathExists( HKEY hKey,PTCHAR ptcPath );
        BOOL  RegistryPathExists( HKEY hKey,PTCHAR *ptcPaths,int nNumPaths );
        BOOL  GetValue( const PTCHAR ptcValueName,VOID *pvValueData,DWORD *pdwDataBytes,DWORD *pdwDataType,int nMaxLen );
        BOOL  GetValue( const PTCHAR ptcValueName,VOID *pvValueData,int nMaxLen );
        BOOL  SetValue( const PTCHAR ptcValueName,VOID *pvValueData,int nDataBytes,DWORD dwDatatype );
        int   GetSubKeys( PTCHAR *ptcKeys,PTCHAR *ptcClasses,int nMaxKeys );
        int   GetAllValues( PTCHAR *ptcValueNames,PDWORD pdwValueTypes,PBYTE *puchData,PDWORD pdwDataBytes,int nMaxNumValues );
        BOOL  DeleteKey( const PTCHAR ptcKeyName );
        BOOL  DeleteValue( const PTCHAR ptcValueName );
        VOID  SetReadOnly( BOOL bVal ) { m_bReadOnly = bVal; }

        BOOL  SetValueBinary( const PTCHAR ptcName,PVOID pData,int nBytes ) \
                        {return( SetValue( ptcName, (PVOID)pData, (DWORD)nBytes, REG_BINARY ) );}
        inline BOOL  SetValueInt( const PTCHAR pName,int nValue) \
                        {return( SetValue( pName, (VOID *)&nValue, sizeof( DWORD ), REG_DWORD ) );}
        inline BOOL  SetValueDWORD( const PTCHAR pName,DWORD dwValue) \
                        {return( SetValue( pName, (VOID *)&dwValue, sizeof( DWORD ), REG_DWORD ) );}
        inline BOOL  SetValueString( const PTCHAR pName,PTCHAR pStr ) \
                        {return( SetValue( pName, (VOID *)pStr,(int)(::_tcsclen( pStr ) * sizeof( TCHAR )) + 2, REG_SZ ) );}
        inline DWORD GetLastError() { return( m_dwRegError ); }

    private:

        BOOL        m_bRegistryOpen; // Registry key is open.
        BOOL        m_bReadOnly;     // Open for read only access
        int         m_nPathLevels;   // Number of levels in registry path.
        DWORD       m_dwRegError;    // Our defined error.
        DWORD       m_dwSysError;    // Windows error number.
        HKEY        m_hkMainRegKey;  // Key under which to open path.
        HKEY        m_hkRegistry;    // Open registry handle.
        PTCHAR      m_chPaths[ MAX_PATH_DEPTH ]; // Path under key.
#ifdef DEBUG
        TCHAR       m_tcDebugStr[ 128 ];
#endif

        BOOL    OpenRegistryPath( VOID );
        VOID    DeleteCurrentPath( VOID );
        int     ParsePath( PTCHAR paths[ MAX_PATH_DEPTH ],PTCHAR ptcPath );
    };

#endif

// $Log: Library_Tools/CRegistry.h  $
// Revision 1.2 2008/01/15 09:37:09EST MontoroJ 
// - Cleaned up some warnings by adding type casting to ::strlen() calls.
// Revision 1.4 2007/04/25 10:09:24EDT MontoroJ 
// - Added include of stdio.h for desktop builds.
// Revision 1.3 2006/11/22 11:39:53EST MontoroJ 
// - Bug fixes and added features.
// Revision 1.2 2004/09/17 15:20:34EDT montoroj 
// - Added variable names to prototypes.
// - Added method for determining whether a registry key exists.
// Revision 1.1 2003/06/19 11:54:34EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// Revision 1.1 2003/05/14 11:39:13EDT montoroj 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.1 2003/05/13 14:50:51EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.3 2003/02/20 16:32:54EST montoroj 
// - Replaced $LOG$ with $Log: Library_Tools/CRegistry.h  $
// - Replaced Revision 1.2 2008/01/15 09:37:09EST MontoroJ 
// - Replaced - Cleaned up some warnings by adding type casting to ::strlen() calls.
// - Replaced Revision 1.4 2007/04/25 10:09:24EDT MontoroJ 
// - Replaced - Added include of stdio.h for desktop builds.
// - Replaced Revision 1.3 2006/11/22 11:39:53EST MontoroJ 
// - Replaced - Bug fixes and added features.
// - Replaced Revision 1.2 2004/09/17 15:20:34EDT montoroj 
// - Replaced - Added variable names to prototypes.
// - Replaced - Added method for determining whether a registry key exists.
// - Replaced Revision 1.1 2003/06/19 11:54:34EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// - Replaced Revision 1.1 2003/05/14 11:39:13EDT montoroj 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// - Replaced Revision 1.1 2003/05/13 14:50:51EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
