/*
******************************************************************************
*
*              Copyright (c) 2001 Hand Held Products Inc.
*
*                        All Rights Reserved.
*
******************************************************************************
*
* $RCSfile: Library_Tools/MemExcpt.c $
* $Revision: 1.1 $
* $Date: 2007/04/24 08:33:19EDT $
* $Author: MontoroJ $
*
*  File:        MemExcpt.c
*
*  Description: Contains function IsValidAddress() which can be used to validate
*               pointer addresses.  It checks for NULL (All OS) and access
*               exceptions (WinNT,Windows & WinCE only).
*
*  Functions:   Public Interface
*               ----------------
*               IsValidAddress()
*
*               Private Interface
*               -----------------
*               MemoryExceptionHandler()  // Not for non-Windows environments.
*
*  Rev. Date    By     Revision Notes
*  ----------   ---    ----------------
*  05/31/2001   JCM    Original Release
*
******************************************************************************
*/
// Include windows header if windows environment
#if defined(_WIN32_WCE) || (WIN32)
#include "Windows.h"
#endif
// Header for this file.
#include "MemExcpt.h"

// Windows family (including CE) only code.
#if defined(_WIN32_WCE) || (WIN32)

/*
******************************************************************************
*
*  Name:        MemoryExceptionHandler()
*
*  Description: Called by __except() from __try when exception occurs.  It
*               causes the exception code to be run only if the exception is
*               STATUS_ACCESS_VIOLATION (0xc0000005). Otherwise, the exception
*               is passed down to the windows handler.
*
*  Parameters:  EXCEPTION_POINTERS *pe; (I)  // Exception information
*
*  Returns:     EXCEPTION_EXECUTE_HANDLER - If STATUS_ACCESS_VIOLATION.
*               EXCEPTION_CONTINUE_SEARCH - Otherwise.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  06/31/2001   JCM   Original Release
*
******************************************************************************
*/

UINT MemoryExceptionHandler( EXCEPTION_POINTERS  *pe )
{
	PEXCEPTION_RECORD per = pe->ExceptionRecord;
    UINT    uRetVal = EXCEPTION_CONTINUE_SEARCH;

	if( per->ExceptionCode == STATUS_ACCESS_VIOLATION )
        uRetVal = EXCEPTION_EXECUTE_HANDLER;

    return( uRetVal );
}
// End MemoryExceptionHandler()

/*
******************************************************************************
*
*  Name:        IsValidAddress()
*
*  Description: Verifies pointer is not NULL.  Then attempts to read first
*               location pointed to by pointer.  Returns error code if either
*               operation fails.
*
*  Parameters:  PVOID pVoid; (I)        // Address to verify. (as void pointer)
*
*  Returns:     RESULT_SUCCESS            - If address is valid.
*               RESULT_ERR_NULLPOINTER    - If pointer is NULL.
*               RESULT_ERR_INVALIDADDRESS - If address pointed to is invalid.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  06/31/2001   JCM   Original Release
*
******************************************************************************
*/

LONG IsValidAddress( PVOID pVoid )
{
    LONG    retVal = RESULT_ADDRESS_OKAY;// Return value.
    PBYTE   pByte = (PBYTE)pVoid;   // Local copy of pointer.
    BYTE    uch = 0;                // Value to receive read byte.
    BYTE    uchTmp = 0;             // Address to point to so execution can
                                        // continue.
    // Verify not NULL
    if( pVoid == NULL )
        retVal = RESULT_ADDRESS_NULL;
    else
    {
        // Attempt to read byte at address. If this throws an exception, it will be
        // caught by the __except() block.
        __try
        {
            uch = *pByte;
        }

        // Called if __try block throws an exception. I calls our handler with the
        // exception information.  If the handler returns EXCEPTION_EXECUTE_HANDLER
        // the block under __except() is executed.  Otherwise the default windows
        // exception handler takes over.
        __except( MemoryExceptionHandler( GetExceptionInformation() ) )
        {
            pByte = &uchTmp;
            retVal = RESULT_ADDRESS_INVALID;
        }
    }

    return( retVal );
}
// End IsValidAddress()

#else
// Non-Windows family code.
/*
******************************************************************************
*
*  Name:        IsValidAddress()
*
*  Description: Verifies pointer is not NULL.  Returns error code if operation
*               fails.
*
*  Parameters:  PVOID pVoid; (I)        // Address to verify. (as void pointer)
*
*  Returns:     RESULT_SUCCESS            - If address is valid.
*               RESULT_ERR_NULLPOINTER    - If pointer is NULL.
*
*  Rev. Date    By    Revision Notes
*  ----------   ---   ----------------
*  06/31/2001   JCM   Original Release
*
******************************************************************************
*/

Result_t IsValidAddress( PVOID pVoid )
{
    Result_t    retVal = RESULT_SUCCESS;

    if( pVoid == NULL )
        retVal = RESULT_ADDRESS_NULL;

    return( retVal );
}
// End IsValidAddress()

#endif

// $Log: Library_Tools/MemExcpt.c  $
// Revision 1.1 2007/04/24 08:33:19EDT MontoroJ 
// Initial revision
// Member added to project d:/MKS_ES/ES/Matrix SDK/Matrix5080SDK/Matrix5080SDK.pj
// Revision 1.1 2003/06/19 11:54:35EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// Revision 1.1 2003/05/14 11:39:14EDT montoroj 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.1 2003/05/13 14:50:51EDT holzhaud 
// Initial revision
// Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// Revision 1.3 2003/02/20 16:33:40EST montoroj 
// - Replaced $LOG$ with $Log: Library_Tools/MemExcpt.c  $
// - Replaced Revision 1.1 2007/04/24 08:33:19EDT MontoroJ 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_ES/ES/Matrix SDK/Matrix5080SDK/Matrix5080SDK.pj
// - Replaced Revision 1.1 2003/06/19 11:54:35EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_IT/SDK/Matrix_SDK_Examples/Matrix_SDK_Examples.pj
// - Replaced Revision 1.1 2003/05/14 11:39:14EDT montoroj 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj
// - Replaced Revision 1.1 2003/05/13 14:50:51EDT holzhaud 
// - Replaced Initial revision
// - Replaced Member added to project d:/MKS_Common/Library_Tools/Library_Tools.pj

